﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//      This file contains a programming example for Intel 82C54 ICs
//      used on ALLDAQ Data Acquisition Boards
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using System.Collections.Generic;

namespace Counter8254.Utility
{
    public static class AdqCommon
    {
        public const uint NotOpen = AdqErrors.ADQ_ERROR_TYPE_FATAL | AdqErrors.ADQ_ERROR_NOT_OPEN;
        public static readonly uint[] Counter8254Modes = {AdqConstants.ADQ_CNTR_8254_MODE_0,
                                                          AdqConstants.ADQ_CNTR_8254_MODE_1,
                                                          AdqConstants.ADQ_CNTR_8254_MODE_2,
                                                          AdqConstants.ADQ_CNTR_8254_MODE_3,
                                                          AdqConstants.ADQ_CNTR_8254_MODE_4,
                                                          AdqConstants.ADQ_CNTR_8254_MODE_5};

        public static string GetCounterClockSourceName(uint counterClockSource)
        {
            return GetDictionaryValue(_counterClockSourceNames, counterClockSource);
        }

        private static string GetDictionaryValue(IDictionary<uint, string> dictionary, uint key)
        {
            bool found = dictionary.TryGetValue(key, out string stringValue);
            if (!found)
            {
                stringValue = string.Format("0x{0:X8}", key);
            }
            return stringValue;
        }

        private static readonly Dictionary<uint, string> _counterClockSourceNames = new Dictionary<uint, string>
        {
            {AdqConstants.ADQ_CNTR_CLOCK_SOURCE_DEFAULT, "Default"},
            {AdqConstants.ADQ_CNTR_CLOCK_SOURCE_EXTERNAL, "External"},
            {AdqConstants.ADQ_CNTR_CLOCK_SOURCE_INTERNAL_10_MHZ, "Internal 10 MHz"},
            {AdqConstants.ADQ_CNTR_CLOCK_SOURCE_INTERNAL_66_MHZ, "Internal 66 MHz"}
        };

        public static string GetModelName(string boardName)
        {
            int posADQ = boardName.IndexOf("ADQ");
            if (posADQ < 0) posADQ = 0;

            int posEnd = boardName.IndexOf(" ", posADQ);
            if (posEnd < 0)
            {
                posEnd = boardName.Length;
            }

            return boardName.Substring(posADQ, posEnd - posADQ);
        }
    }
}
