﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the DISingle project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using System.Collections.Generic;
using System.Linq;

namespace DISingle.Utility
{
    static class AdqCommon
    {
        public const uint NotOpen = AdqErrors.ADQ_ERROR_TYPE_FATAL | AdqErrors.ADQ_ERROR_NOT_OPEN;
        private static readonly string[] _binStrings = {"0000", "0001", "0010", "0011",
                                                      "0100", "0101", "0110", "0111",
                                                      "1000", "1001", "1010", "1011",
                                                      "1100", "1101", "1110", "1111"};
        private static readonly uint[] _bitCounts = { 1, 8, 16, 24, 32 };

        public static string GetModuleTypeName(uint moduleType)
        {
            return GetDictionaryValue(_moduleTypeNames, moduleType);
        }

        private static string GetDictionaryValue(IDictionary<uint, string> dictionary, uint key)
        {
            bool found = dictionary.TryGetValue(key, out string stringValue);
            if (!found)
            {
                stringValue = string.Format("0x{0:X8}", key);
            }
            return stringValue;
        }

        private static readonly Dictionary<uint, string> _moduleTypeNames = new Dictionary<uint, string>
        {
            {AdqConstants.ADQ_MODULE_TYPE_DIO, "DIO"},
            {AdqConstants.ADQ_MODULE_TYPE_DO, "DO"},
            {AdqConstants.ADQ_MODULE_TYPE_DI, "DI"},
            {AdqConstants.ADQ_MODULE_TYPE_CNTR, "Counter"},
            {AdqConstants.ADQ_MODULE_TYPE_EXTINT, "External Interrupt"},
            {AdqConstants.ADQ_MODULE_TYPE_AI, "AI"},
            {AdqConstants.ADQ_MODULE_TYPE_AO, "AO"},
            {AdqConstants.ADQ_MODULE_TYPE_I2C, "I2C"},
            {AdqConstants.ADQ_MODULE_TYPE_PWM, "PWM"},
            {AdqConstants.ADQ_MODULE_TYPE_FIN, "FIN"},
            {AdqConstants.ADQ_MODULE_TYPE_I2SO, "I2SO"}
        };

        public static string IntToBin(uint val, uint bitCount)
        {
            string tmpStr = "";

            if (_bitCounts.Contains(bitCount))
            {
                if (bitCount == 1)
                {
                    tmpStr = (val & 1).ToString();
                }
                else
                {
                    for (uint i = (bitCount / 4); i > 0; i--)
                    {
                        uint binIdx = (val >> ((int)(i - 1) * 4)) & 0x0F;
                        tmpStr += _binStrings[binIdx];
                    }
                }
            }
            return tmpStr;
        }

        public static string GetModelName(string boardName)
        {
            int posADQ = boardName.IndexOf("ADQ");
            if (posADQ < 0) posADQ = 0;

            int posEnd = boardName.IndexOf(" ", posADQ);
            if (posEnd < 0)
            {
                posEnd = boardName.Length;
            }

            return boardName.Substring(posADQ, posEnd - posADQ);
        }
    }
}
