﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the Interrupt project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using System.Collections.Generic;

namespace Interrupt.Utility
{
    public static class AdqCommon
    {
        public static readonly uint NotOpen = AdqErrors.ADQ_ERROR_TYPE_FATAL | AdqErrors.ADQ_ERROR_NOT_OPEN;
        public static readonly uint DefaultIrqWaitTimeoutMilliseconds = 5000;

        public static string GetModuleTypeName(uint moduleType)
        {
            return GetDictionaryValue(_moduleTypeNames, moduleType);
        }

        private static string GetDictionaryValue(IDictionary<uint, string> dictionary, uint key)
        {
            bool found = dictionary.TryGetValue(key, out string stringValue);
            if (!found)
            {
                stringValue = string.Format("0x{0:X8}", key);
            }
            return stringValue;
        }

        private static readonly Dictionary<uint, string> _moduleTypeNames = new Dictionary<uint, string>
        {
            {AdqConstants.ADQ_MODULE_TYPE_DIO, "DIO"},
            {AdqConstants.ADQ_MODULE_TYPE_DO, "DO"},
            {AdqConstants.ADQ_MODULE_TYPE_DI, "DI"},
            {AdqConstants.ADQ_MODULE_TYPE_CNTR, "Counter"},
            {AdqConstants.ADQ_MODULE_TYPE_EXTINT, "External Interrupt"},
            {AdqConstants.ADQ_MODULE_TYPE_AI, "AI"},
            {AdqConstants.ADQ_MODULE_TYPE_AO, "AO"},
            {AdqConstants.ADQ_MODULE_TYPE_I2C, "I2C"},
            {AdqConstants.ADQ_MODULE_TYPE_PWM, "PWM"},
            {AdqConstants.ADQ_MODULE_TYPE_FIN, "FIN"},
            {AdqConstants.ADQ_MODULE_TYPE_I2SO, "I2SO"}
        };

        public static string GetModelName(string boardName)
        {
            int posADQ = boardName.IndexOf("ADQ");
            if (posADQ < 0) posADQ = 0;

            int posEnd = boardName.IndexOf(" ", posADQ);
            if (posEnd < 0)
            {
                posEnd = boardName.Length;
            }

            return boardName.Substring(posADQ, posEnd - posADQ);
        }

        public static string FormatHexValue(uint value)
        {
            return value.ToString("X");
        }
    }
}
