﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the Interrupt project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------


namespace Interrupt.ViewModels
{
    public class PortStatusItemViewModel : ViewModelBase
    {
        public PortStatusItemViewModel(int bitNumber, bool statusErrorEnabled, bool faultRecoveryEnabled)
        {
            BitNumber = bitNumber;
            BinaryValue = (uint)(1 << bitNumber);
            StatusErrorSupported = statusErrorEnabled;
            FaultRecoverySupported = faultRecoveryEnabled;
        }

        public int BitNumber { get; }
        public uint BinaryValue { get; }
        public bool StatusErrorSupported { get; }

        private bool _bitStatusError;
        public bool BitStatusError
        {
            get { return _bitStatusError; }
            set
            {
                if (_bitStatusError != value)
                {
                    _bitStatusError = value;
                    NotifyPropertyChanged(nameof(BitStatusError));
                }
            }
        }

        public bool FaultRecoverySupported { get; }

        private bool _bitFaultRecovery;
        public bool BitFaultRecovery
        {
            get { return _bitFaultRecovery; }
            set
            {
                if (_bitFaultRecovery != value)
                {
                    _bitFaultRecovery = value;
                    NotifyPropertyChanged(nameof(BitFaultRecovery));
                }
            }
        }
    }
}
