﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the ReadBuffer project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ReadBuffer.Utility;
using System.Text;

namespace ReadBuffer.ALLDAQ
{
    public class AnalogRange
    {
        private readonly double _physicalOffset;

        public string RangeText { get; private set; }
        public string UnitSymbol { get; private set; }
        public string UnitName { get; private set; }

        public AnalogRange(uint physicalUnit, uint digitalMin, uint digitalMax, double physicalMin, double physicalMax)
        {
            PhysicalResolution = (physicalMax - physicalMin) / (digitalMax - digitalMin);
            _physicalOffset = physicalMin - digitalMin * PhysicalResolution;

            UnitSymbol = AdqCommon.GetPhysicalUnitSymbol(physicalUnit);
            UnitName = AdqCommon.GetPhysicalUnitName(physicalUnit);
            SetRangeUnitTexts(physicalMin, physicalMax);
        }

        public double PhysicalResolution { get; }

        public double DigitalToPhysical(uint digitalValue)
        {
            return digitalValue * PhysicalResolution + _physicalOffset;
        }

        private void SetRangeUnitTexts(double physicalMin, double physicalMax)
        {
            StringBuilder rangeTxt = new StringBuilder();

            rangeTxt.Append(AdqCommon.FormatPhysicalValue(physicalMin));
            rangeTxt.Append(UnitSymbol);
            rangeTxt.Append("...");
            rangeTxt.Append(AdqCommon.FormatPhysicalValue(physicalMax));
            rangeTxt.Append(UnitSymbol);

            RangeText = rangeTxt.ToString();
        }
    }
}
