﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the ReadBuffer project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using ReadBuffer.Utility;
using System.Collections.Generic;

namespace ReadBuffer.ALLDAQ
{
    public class CounterModule : InputModule
    {
        public CounterModule(string boardName, uint boardIndex, uint moduleIndex, 
            ref AdqDriver.ADQGeneralModuleInfo moduleInfo, ref AdqDriver.ADQCNTRModuleInfo cntrModuleInfo)
            : base(boardName, boardIndex, moduleIndex, ref moduleInfo)
        {
            SetClockSourceValues(cntrModuleInfo.ClockSources);
            SelectedStartValue = (uint)((1 << (int)cntrModuleInfo.WidthBits) -1);
            BuildModuleName();
        }

        private void BuildModuleName()
        {
            _moduleName = string.Format("{0}, Board {1}, Module {2}, Type {3}, {4} clock, max. {5}",
                _modelName,
                _boardIndex,
                _moduleIndex,
                AdqCommon.GetModuleTypeName(_moduleType),
                AdqCommon.GetClockType(_clockType),
                AdqCommon.FormatFrequency(MaxfrequencyHz));
        }

        public List<ClockSourceItem> ClockSourceItems { get; } = new List<ClockSourceItem>();

        private void SetClockSourceValues(uint clockSources)
        {
            ClockSourceItems.Clear();
            if ((clockSources & AdqConstants.ADQ_CNTR_CLOCK_SOURCE_EXTERNAL) == AdqConstants.ADQ_CNTR_CLOCK_SOURCE_EXTERNAL)
            {
                ClockSourceItems.Add(new ClockSourceItem(AdqConstants.ADQ_CNTR_CLOCK_SOURCE_EXTERNAL, 
                    AdqCommon.GetCounterClockSourceName(AdqConstants.ADQ_CNTR_CLOCK_SOURCE_EXTERNAL)));
            }

            if ((clockSources & AdqConstants.ADQ_CNTR_CLOCK_SOURCE_INTERNAL_10_MHZ) == AdqConstants.ADQ_CNTR_CLOCK_SOURCE_INTERNAL_10_MHZ)
            {
                ClockSourceItems.Add(new ClockSourceItem(AdqConstants.ADQ_CNTR_CLOCK_SOURCE_INTERNAL_10_MHZ, 
                    AdqCommon.GetCounterClockSourceName(AdqConstants.ADQ_CNTR_CLOCK_SOURCE_INTERNAL_10_MHZ)));
            }
        }

        public int SelectedClockSourceIndex { get; private set; }
        public int SelectedCounterModeIndex { get; private set; }
        public uint SelectedStartValue { get; private set; }

        internal uint ConfigureModule(int clockSourceIndex, int counterModeIndex, uint startValue)
        {
            uint clockSource = ClockSourceItems[clockSourceIndex].ClockSourceValue;
            uint counterMode = AdqCommon.Counter8254Modes[counterModeIndex];

            uint adqError = AdqDriver.adqCNTRConfig(_boardIndex, _moduleIndex, clockSource, counterMode, startValue);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                ErrorTextChanged("adqCNTRConfig", adqError);
            }
            else
            {
                SelectedClockSourceIndex = clockSourceIndex;
                SelectedCounterModeIndex = counterModeIndex;
                SelectedStartValue = startValue;
            }

            return adqError;
        }
    }
}
