﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the ReadBuffer project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using ReadBuffer.Utility;
using System;
using System.Runtime.InteropServices;

namespace ReadBuffer.ALLDAQ
{
    public class DigitalInputModule : InputModule
    {
        public DigitalInputModule(string boardName, uint boardIndex, uint moduleIndex, 
            ref AdqDriver.ADQGeneralModuleInfo moduleInfo, uint portWidthBits)
            : base(boardName, boardIndex, moduleIndex, ref moduleInfo)
        {
            PortWidthBits = portWidthBits;
            BuildModuleName();
        }

        public uint PortWidthBits { get; }

        private void BuildModuleName()
        {
            _moduleName = string.Format("{0}, Board {1}, Module {2}, Type {3}, {4} Bit, {5} clock, max. {6}",
                _modelName,
                _boardIndex,
                _moduleIndex,
                AdqCommon.GetModuleTypeName(_moduleType),
                PortWidthBits,
                AdqCommon.GetClockType(_clockType),
                AdqCommon.FormatFrequency(MaxfrequencyHz));
        }

        internal uint ConfigureModule()
        {
            if (_moduleType == AdqConstants.ADQ_MODULE_TYPE_DIO)
            {
                uint adqError = AdqDriver.adqDIOSetDirection(_boardIndex, _moduleIndex, AdqConstants.ADQ_DIO_INPUT);
                if (adqError != AdqErrors.ADQ_ERROR_NONE)
                {
                    ErrorTextChanged("adqDIOSetDirection", adqError);
                    return adqError;
                }
            }
            return AdqErrors.ADQ_ERROR_NONE;
        }
    }
}
