//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the ReadBuffer project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ReadBuffer.Utility;
using System;

namespace ReadBuffer.ALLDAQ
{
    public interface IInputModule
    {
        string ModelName { get; }
        uint BoardIndex { get; }
        uint ModuleIndex { get; }
        uint ModuleType { get; }
        uint ClockType { get; }
        double MaxfrequencyHz { get; }
        double DesiredSampleFrequency { get; set; }
        double ActualSampleFrequency { get; }
        double SampleTime { get; }
        string ModuleName { get; }
        uint AdqError { get; set; }
        uint NumberOfValues { get; set; }
        uint NumberOfValuesRead { get; }
        uint[] Values { get; }


        event EventHandler<InfoTextChangedEventArgs> ModuleInfoTextChanged;
        void ValidateConfigurationValues();
        double GetSecondsToWait();
        uint GetModuleStatus(out bool isRunning);
        uint StopAcquisition();
        uint Reset();
        void InitValueBuffer();
        uint SetSampleFrequency();
        void ReadValuesFromModule();
        void ClearValueBuffer();
    }
}