﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferAO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using System;
using WriteBufferAO.Utility;

namespace WriteBufferAO.Models
{
    public class SineCutWaveGenerator : ISignalGenerator
    {
        public string GeneratorIcon
        {
            get
            {
                return "/WriteBufferAO;component/Images/signalSineCut.png";
            }
        }

        public string GeneratorName
        {
            get
            {
                return "Sine cut";
            }
        }

        public double GetValue(double cycleTime, double signalMinimum, double signalMaximum)
        {
            double offset = (signalMaximum + signalMinimum) / 2.0;
            if (cycleTime % 0.5 > 0.25)
                return offset + (signalMaximum - signalMinimum) / 2.0 * Math.Sin(AdqCommon.TwoPI * cycleTime);
            else
                return offset;
        }
    }
}
