﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferAO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using System;

namespace WriteBufferAO.Models
{
    public class WaveForm
    {
        private double _outputFrequency;
        private double _signalLengthSeconds;

        public void InitWaveForm(double outputFrequency, double signalLengthSeconds, uint numberOfValues)
        {
            _outputFrequency = outputFrequency;
            _signalLengthSeconds = signalLengthSeconds;
            if (_values == null || _values.Length != numberOfValues)
            {
                _values = new double[numberOfValues];
            }
            Array.Clear(_values, 0, _values.Length);
        }

        private double[] _values;
        public double[] Values
        {
            get { return _values; }
        }

        public void ApplySignalGenerator(ISignalGenerator signalGenerator, SignalSettings signalSettings)
        {
            double frequencyShift = (signalSettings.SignalEndFrequency - signalSettings.SignalBeginFrequency) / _signalLengthSeconds;
            double physicalMinShift = (signalSettings.SignalEndPhysicalMin - signalSettings.SignalBeginPhysicalMin) / _signalLengthSeconds;
            double physicalMaxShift = (signalSettings.SignalEndPhysicalMax - signalSettings.SignalBeginPhysicalMax) / _signalLengthSeconds;

            for (int i = 0; i < _values.Length; i++)
            {
                double time = i / _outputFrequency;
                double signalFrequency = signalSettings.SignalBeginFrequency + time * frequencyShift;
                double signalPhysicalMin = signalSettings.SignalBeginPhysicalMin + time * physicalMinShift;
                double signalPhysicalMax = signalSettings.SignalBeginPhysicalMax + time * physicalMaxShift;
                double timeOffset = time * signalFrequency + signalSettings.PhaseShift;
                double cycleTime = timeOffset - Math.Floor(timeOffset);

                double currValue = _values[i] + signalGenerator.GetValue(cycleTime, signalPhysicalMin, signalPhysicalMax);

                if (currValue < signalSettings.RangePhysicalMin)
                {
                    currValue = signalSettings.RangePhysicalMin;
                }
                else if (currValue > signalSettings.RangePhysicalMax)
                {
                    currValue = signalSettings.RangePhysicalMax;
                }

                _values[i] = currValue;
            }
        }
    }
}
