﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferAO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------


namespace WriteBufferAO.Utility
{
    public static class InputValidation
    {
        public static string ValidateValueIsWithinRange(string textValue, double minLimit, double maxLimit, out double doubleValue)
        {
            doubleValue = 0.0;
            string msg = ValidateDoubleValue(textValue, out double value);
            if (string.IsNullOrEmpty(msg))
            {
                if (value < minLimit || value > maxLimit)
                {
                    msg = string.Format("The entered value {2} is outside the limits ({0}...{1})", minLimit, maxLimit, value);
                }
                else
                {
                    doubleValue = value;
                }
            }
            return msg;
        }

        public static string ValidateValueIsPositive(string textValue, out double doubleValue)
        {
            doubleValue = 0.0;
            string msg = ValidateDoubleValue(textValue, out double value);
            if (string.IsNullOrEmpty(msg))
            {
                if (value <= 0)
                {
                    msg = "The value must be a positive number";
                }
                else
                {
                    doubleValue = value;
                }
            }
            return msg;
        }

        private static string ValidateDoubleValue(string textValue, out double doubleValue)
        {
            doubleValue = 0.0;
            string msg = null;
            if (string.IsNullOrEmpty(textValue))
            {
                msg = "The value cannot be empty";
            }
            else if (!double.TryParse(textValue, out doubleValue))
            {
                msg = "The entered value \"" + textValue + "\" is not a valid number";
            }
            return msg;
        }
    }
}
