﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the AIReadContinuous project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.Text

Public Class AnalogRange
    Private ReadOnly _physicalOffset As Double

    Public Sub New(rangeIndex As UInteger, physicalUnit As UInteger, digitalMin As UInteger, digitalMax As UInteger, physicalMin As Double, physicalMax As Double)
        Me.RangeIndex = rangeIndex

        PhysicalResolution = (physicalMax - physicalMin) / (digitalMax - digitalMin)
        _physicalOffset = physicalMin - digitalMin * PhysicalResolution

        UnitSymbol = AdqCommon.GetPhysicalUnitSymbol(physicalUnit)
        UnitName = AdqCommon.GetPhysicalUnitName(physicalUnit)
        RangeText = GetRangeText(physicalMin, physicalMax)
    End Sub

    Public ReadOnly Property PhysicalResolution As Double
    Public ReadOnly Property RangeIndex As UInteger
    Public ReadOnly Property UnitSymbol As String
    Public ReadOnly Property UnitName As String
    Public ReadOnly Property RangeText As String

    Public Function DigitalToPhysical(digitalValue As UInteger) As Double
        Return digitalValue * PhysicalResolution + _physicalOffset
    End Function

    Private Function GetRangeText(physicalMin As Double, physicalMax As Double) As String
        Dim rangeTxt = New StringBuilder()

        rangeTxt.Append(RangeIndex)
        rangeTxt.Append(": ")
        rangeTxt.Append(AdqCommon.FormatPhysicalValue(physicalMin))
        rangeTxt.Append(UnitSymbol)
        rangeTxt.Append("...")
        rangeTxt.Append(AdqCommon.FormatPhysicalValue(physicalMax))
        rangeTxt.Append(UnitSymbol)
        Return rangeTxt.ToString()
    End Function
End Class
