﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the AOSingle project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.ComponentModel
Imports AOSingle.ALLDAQ.Boards

Public Class AoModule
    Implements INotifyPropertyChanged

    Public Event PropertyChanged As PropertyChangedEventHandler _
        Implements INotifyPropertyChanged.PropertyChanged

    Public Sub New(boardName As String, boardIndex As UInteger, moduleIndex As UInteger,
                   ByRef aoModuleInfo As AdqDriver.ADQAOModuleInfo)
        Me.BoardIndex = boardIndex
        Me.ModuleIndex = moduleIndex
        ModuleName = String.Format("{0}, Board {1}, Module {2}", AdqCommon.GetModelName(boardName), boardIndex, moduleIndex)
        AddRanges(aoModuleInfo)
    End Sub

    Private Sub AddRanges(ByRef aoModuleInfo As AdqDriver.ADQAOModuleInfo)

        Dim rangeIdx As UInteger = 0
        While rangeIdx < aoModuleInfo.NumberOfRanges
            Dim adqRange As AdqDriver.ADQAORange = aoModuleInfo.Ranges(CInt(rangeIdx))
            Dim aoRange = New AoRange(rangeIdx,
                    adqRange.DigitalMin,
                    adqRange.DigitalMax,
                    adqRange.PhysicalUnit,
                    adqRange.PhysicalMin,
                    adqRange.PhysicalMax)
            AoRanges.Add(aoRange)
            rangeIdx += 1UI
        End While

        If aoModuleInfo.NumberOfRanges >= 1 Then
            SelectedAoRange = AoRanges(0)
        End If
    End Sub

    Public Property SelectedAoRange As AoRange
    Public ReadOnly Property AoRanges As List(Of AoRange) = New List(Of AoRange)
    Public ReadOnly Property BoardIndex As UInteger
    Public ReadOnly Property ModuleIndex As UInteger
    Public ReadOnly Property ModuleName As String

    Public Function ResetModule() As Boolean
        Dim adqError As UInteger = AdqDriver.adqResetModule(BoardIndex, ModuleIndex)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqResetModule", adqError)
            Return False
        End If
        Return True
    End Function

    Public Function WriteSingleValue(digitalValue As UInteger) As Boolean
        Dim adqError As UInteger = AdqDriver.adqWriteSingle(BoardIndex, ModuleIndex, digitalValue)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqWriteSingle", adqError)
            Return False
        End If
        Return True
    End Function

    Private Sub SetFunctionErrorInfo(functionName As String, adqError As UInteger)
        ModuleInfoText = String.Format("Board {0}, Module {1}: API-function {2} returned error number: 0x{3:X}",
                BoardIndex,
                ModuleIndex,
                functionName,
                adqError)
    End Sub

    Private _infoText As String = String.Empty
    Public Property ModuleInfoText As String
        Get
            Return _infoText
        End Get
        Set
            _infoText = Value
            If Value <> String.Empty Then
                NotifyPropertyChanged("ModuleInfoText")
            End If
        End Set
    End Property

    Public Sub NotifyPropertyChanged(ByVal propertyName As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
    End Sub
End Class
