﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the AOSingle project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports AOSingle.ALLDAQ.Boards

Public Class AdqCommon
    Public Shared ReadOnly NotOpen As UInteger = AdqErrors.ADQ_ERROR_TYPE_FATAL Or AdqErrors.ADQ_ERROR_NOT_OPEN

    Public Shared Function GetPhysicalUnitSymbol(physicalUnit As UInteger) As String
        Return GetDictionaryValue(_physicalUnitSymbols, physicalUnit)
    End Function

    Private Shared Function GetDictionaryValue(dictionary As IDictionary(Of UInteger, String), key As UInteger) As String
        Dim stringValue As String = String.Empty
        Dim found As Boolean = dictionary.TryGetValue(key, stringValue)
        If Not found Then
            stringValue = String.Format("&H{0:X8}", key)
        End If
        Return stringValue
    End Function

    Private Shared ReadOnly _physicalUnitSymbols As Dictionary(Of UInteger, String) =
            New Dictionary(Of UInteger, String) From
            {
                {AdqConstants.ADQ_PHYSICAL_UNIT_VOLT, "V"},
                {AdqConstants.ADQ_PHYSICAL_UNIT_AMPERE, "A"},
                {AdqConstants.ADQ_PHYSICAL_UNIT_OHM, "Ω"}
            }

    Public Shared Function GetModelName(boardName As String) As String
        Dim posADQ As Integer = boardName.IndexOf("ADQ")
        If posADQ < 0 Then
            posADQ = 0
        End If

        Dim posEnd As Integer = boardName.IndexOf(" ", posADQ)
        If posEnd < 0 Then
            posEnd = boardName.Length
        End If

        Return boardName.Substring(posADQ, posEnd - posADQ)
    End Function

    Public Shared Function FormatDoubleValue(physicalValue As Double) As String
        Return String.Format("{0:0.######}", physicalValue)
    End Function

    Public Shared Function FormatPhysicalValue(physicalValue As Double) As String
        Dim physicalTxt As String
        Dim tmpVal As Double = Math.Abs(physicalValue)
        If tmpVal = 0 Then
            physicalTxt = "0"
        ElseIf tmpVal < 0.001 Then
            physicalTxt = String.Format("{0:0.######}µ", physicalValue * 1000000)
        ElseIf tmpVal < 1 Then
            physicalTxt = String.Format("{0:0.######}m", physicalValue * 1000)
        Else
            physicalTxt = String.Format("{0:0.######}", physicalValue)
        End If
        Return physicalTxt
    End Function
End Class
