﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the CommonSettings project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports CommonSettings.ALLDAQ.Boards

Public Class AdqModule

    Sub New(boardIndex As UInteger, moduleIndex As UInteger)
        Me.BoardIndex = boardIndex
        Me.ModuleIndex = moduleIndex
    End Sub

    Public Event InfoTextChanged As EventHandler(Of InfoTextChangedEventArgs)
    Public ReadOnly Property BoardIndex As UInteger
    Public ReadOnly Property ModuleIndex As UInteger

    Public Function Reset() As Boolean
        Dim adqError As UInteger = AdqDriver.adqResetModule(BoardIndex, ModuleIndex)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqResetModule", adqError)
        End If
        Return adqError = AdqErrors.ADQ_ERROR_NONE
    End Function

    Protected Sub SetFunctionErrorInfo(functionName As String, adqError As UInteger)
        RaiseOnInfoTextChanged(String.Format("API-function {0} returned Error number: &H{1:X}", functionName, adqError))
    End Sub

    Protected Sub RaiseOnInfoTextChanged(infoText As String)
        If InfoTextChangedEvent IsNot Nothing Then
            Dim args = New InfoTextChangedEventArgs(infoText)
            RaiseEvent InfoTextChanged(Me, args)
        End If
    End Sub
End Class
