﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file is part of the CommonSettings project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports CommonSettings.ALLDAQ.Boards

Public Class AdqCommon
    Private Shared ReadOnly _formatStr As String = "{0:0.######}"

    Public Shared ReadOnly NotOpen As UInteger = AdqErrors.ADQ_ERROR_TYPE_FATAL Or AdqErrors.ADQ_ERROR_NOT_OPEN

    Public Shared Function GetAdjustmentText(adjustmentId As UInteger) As String
        Return GetDictionaryValue(_calibrationTexts, adjustmentId)
    End Function

    Public Shared Function GetVoltageText(voltage As UInteger) As String
        Return GetDictionaryValue(_voltageTexts, voltage)
    End Function

    Public Shared Function GetModuleTypeText(moduleType As UInteger) As String
        Return GetDictionaryValue(_moduleTypeTexts, moduleType)
    End Function

    Private Shared Function GetDictionaryValue(dictionary As IDictionary(Of UInteger, String), key As UInteger) As String
        Dim stringValue As String = String.Empty
        Dim found As Boolean = dictionary.TryGetValue(key, stringValue)
        If (Not found) Then
            stringValue = String.Format("&H{0:X8}", key)
        End If
        Return stringValue
    End Function

    Private Shared ReadOnly _moduleTypeTexts As Dictionary(Of UInteger, String) = New Dictionary(Of UInteger, String) From
    {
        {AdqConstants.ADQ_MODULE_TYPE_AI, "AI"},
        {AdqConstants.ADQ_MODULE_TYPE_AO, "AO"}
    }

    Private Shared ReadOnly _calibrationTexts As Dictionary(Of UInteger, String) = New Dictionary(Of UInteger, String) From
    {
        {AdqConstants.ADQ_CALIBRATION_FACTORY, "Factory"},
        {AdqConstants.ADQ_CALIBRATION_USER_1, "User"}
    }

    Private Shared ReadOnly _voltageTexts As Dictionary(Of UInteger, String) = New Dictionary(Of UInteger, String) From
    {
        {AdqConstants.ADQ_TTL_VOLTAGE_LEVEL_5_0_VOLTS, String.Format("{0:0.0} V", 5.0)},
        {AdqConstants.ADQ_TTL_VOLTAGE_LEVEL_3_3_VOLTS, String.Format("{0:0.0} V", 3.3)}
    }

    Public Shared Function GetModelName(boardName As String) As String
        Dim posADQ As Integer = boardName.IndexOf("ADQ")
        If posADQ < 0 Then
            posADQ = 0
        End If
        Dim posEnd As Integer = boardName.IndexOf(" ", posADQ)
        If posEnd < 0 Then
            posEnd = boardName.Length
        End If
        Return boardName.Substring(posADQ, posEnd - posADQ)
    End Function

    Public Shared Function FormatDoubleValue(value As Double, unitText As String) As String
        Dim valueStr As String
        Dim unitStr As String = ""
        Dim tmpVal As Double = Math.Abs(value)
        If tmpVal = 0 Then
            valueStr = "0"
        ElseIf tmpVal < 0.000000001 Then
            valueStr = String.Format(_formatStr, value * 1000000000000.0)
            unitStr = "p"
        ElseIf tmpVal < 0.000001 Then
            valueStr = String.Format(_formatStr, value * 1000000000.0)
            unitStr = "n"
        ElseIf tmpVal < 0.001 Then
            valueStr = String.Format(_formatStr, value * 1000000.0)
            unitStr = "µ"
        ElseIf tmpVal < 1 Then
            valueStr = String.Format(_formatStr, value * 1000.0)
            unitStr = "m"
        ElseIf tmpVal >= 1000000000.0 Then
            valueStr = String.Format(_formatStr, value / 1000000000.0)
            unitStr = "G"
        ElseIf tmpVal >= 1000000.0 Then
            valueStr = String.Format(_formatStr, value / 1000000.0)
            unitStr = "M"
        ElseIf tmpVal >= 1000.0 Then
            valueStr = String.Format(_formatStr, value / 1000.0)
            unitStr = "k"
        Else
            valueStr = String.Format(_formatStr, value)
        End If
        Return valueStr & " " & unitStr & unitText
    End Function
End Class
