﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the DISingle project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports DISingle.ALLDAQ.Boards

Public Class DigitalModule
    Private ReadOnly _moduleType As UInteger

    Public Sub New(boardName As String, boardIndex As UInteger, moduleIndex As UInteger,
                   ByRef generalInfo As AdqDriver.ADQGeneralModuleInfo, portWidthBits As UInteger)
        Me.BoardIndex = boardIndex
        Me.ModuleIndex = moduleIndex
        _moduleType = generalInfo.ModuleType
        Description = generalInfo.ModuleDescription
        Me.PortWidthBits = portWidthBits
        ModuleName = GetModuleName(boardName)
    End Sub

    Public Event OnModuleInfoTextChanged As EventHandler(Of InfoTextChangedEventArgs)
    Public ReadOnly Property BoardIndex As UInteger
    Public ReadOnly Property ModuleIndex As UInteger
    Public ReadOnly Property Description As String
    Public ReadOnly Property ModuleName As String
    Public ReadOnly Property PortWidthBits As UInteger

    Public Function ReadSingle(ByRef value As UInteger) As Boolean
        Dim success As Boolean = SetDioDirectionToInput()
        If Not success Then
            Return False
        End If

        Dim adqError As UInteger = AdqDriver.adqReadSingle(BoardIndex, ModuleIndex, value)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqReadSingle", adqError)
        End If
        Return adqError = AdqErrors.ADQ_ERROR_NONE
    End Function

    Private Function SetDioDirectionToInput() As Boolean
        Dim adqError As UInteger = AdqErrors.ADQ_ERROR_NONE
        If _moduleType = AdqConstants.ADQ_MODULE_TYPE_DIO Then
            adqError = AdqDriver.adqDIOSetDirection(BoardIndex, ModuleIndex, AdqConstants.ADQ_DIO_INPUT)
            If (adqError <> AdqErrors.ADQ_ERROR_NONE) Then
                SetFunctionErrorInfo("adqDIOSetDirection", adqError)
            End If
        End If
        Return adqError = AdqErrors.ADQ_ERROR_NONE
    End Function

    Public Function ResetModule() As Boolean
        Dim adqError As UInteger = AdqDriver.adqResetModule(BoardIndex, ModuleIndex)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            SetFunctionErrorInfo("adqResetModule", adqError)
        End If
        Return adqError = AdqErrors.ADQ_ERROR_NONE
    End Function

    Private Function GetModuleName(boardName As String) As String
        Return String.Format("{0}, Board {1}, Module {2}, Type {3}, {4} Bit",
                AdqCommon.GetModelName(boardName),
                BoardIndex,
                ModuleIndex,
                AdqCommon.GetModuleTypeName(_moduleType),
                PortWidthBits)
    End Function

    Private Sub SetFunctionErrorInfo(functionName As String, adqError As UInteger)
        UpdateInfoText(String.Format("API-function {0} returned Error number: &H{1:X}",
                functionName,
                adqError))
    End Sub

    Private Sub UpdateInfoText(text As String)
        Dim args = New InfoTextChangedEventArgs With {
            .InfoText = text
        }
        RaiseOnModuleInfoTextChanged(args)
    End Sub

    Protected Sub RaiseOnModuleInfoTextChanged(e As InfoTextChangedEventArgs)
        RaiseEvent OnModuleInfoTextChanged(Me, e)
    End Sub
End Class
