﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the ReadBuffer project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.Text

Public Class AnalogRange
    Private ReadOnly _physicalOffset As Double

    Public Property RangeText As String
    Public Property UnitSymbol As String
    Public Property UnitName As String

    Public Sub New(physicalUnit As UInteger, digitalMin As UInteger, digitalMax As UInteger, physicalMin As Double, physicalMax As Double)
        PhysicalResolution = (physicalMax - physicalMin) / (digitalMax - digitalMin)
        _physicalOffset = physicalMin - digitalMin * PhysicalResolution

        UnitSymbol = AdqCommon.GetPhysicalUnitSymbol(physicalUnit)
        UnitName = AdqCommon.GetPhysicalUnitName(physicalUnit)
        SetRangeUnitTexts(physicalMin, physicalMax)
    End Sub

    Public ReadOnly Property PhysicalResolution As Double

    Public Function DigitalToPhysical(digitalValue As UInteger) As Double
        Return digitalValue * PhysicalResolution + _physicalOffset
    End Function

    Private Sub SetRangeUnitTexts(physicalMin As Double, physicalMax As Double)
        Dim rangeTxt = New StringBuilder()
        rangeTxt.Append(AdqCommon.FormatPhysicalValue(physicalMin))
        rangeTxt.Append(UnitSymbol)
        rangeTxt.Append("...")
        rangeTxt.Append(AdqCommon.FormatPhysicalValue(physicalMax))
        rangeTxt.Append(UnitSymbol)
        RangeText = rangeTxt.ToString()
    End Sub
End Class
