﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the ReadBuffer project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports ReadBuffer.ALLDAQ.Boards

Public Class CounterModule
    Inherits InputModule

    Public Sub New(boardName As String, boardIndex As UInteger, moduleIndex As UInteger,
                   ByRef generalInfo As AdqDriver.ADQGeneralModuleInfo, ByRef cntrModuleInfo As AdqDriver.ADQCNTRModuleInfo)
        MyBase.New(boardName, boardIndex, moduleIndex, generalInfo)
        SetClockSourceValues(cntrModuleInfo.ClockSources)
        _selectedStartValue = GetMaxStartValue(cntrModuleInfo.WidthBits)

        _moduleName = GetModuleName()
    End Sub

    Private Function GetModuleName() As String
        Return String.Format("{0}, Board {1}, Module {2}, Type {3}, {4} clock, max. {5}",
            ModelName,
            BoardIndex,
            ModuleIndex,
            AdqCommon.GetModuleTypeName(ModuleType),
            AdqCommon.GetClockTypeName(ClockType),
            AdqCommon.FormatFrequency(MaxfrequencyHz))
    End Function

    Private Function GetMaxStartValue(counterWidthBits As UInteger) As UInteger
        If counterWidthBits >= 32 Then
            Return UInteger.MaxValue
        End If
        Return (1UI << CInt(counterWidthBits)) - 1UI
    End Function

    Public ReadOnly Property ClockSourceValues As List(Of ClockSourceItem) = New List(Of ClockSourceItem)

    Private Sub SetClockSourceValues(clockSources As UInteger)
        AddClockSourceItem(clockSources, AdqConstants.ADQ_CNTR_CLOCK_SOURCE_EXTERNAL)
        AddClockSourceItem(clockSources, AdqConstants.ADQ_CNTR_CLOCK_SOURCE_INTERNAL_10_MHZ)
    End Sub

    Private Sub AddClockSourceItem(clockSources As UInteger, clockSourceConstant As UInteger)
        If (clockSources And clockSourceConstant) = clockSourceConstant Then
            ClockSourceValues.Add(New ClockSourceItem(clockSourceConstant,
                                                     AdqCommon.GetCounterClockSourceName(clockSourceConstant)))
        End If
    End Sub

    Private _selectedClockSourceIndex As UInteger
    Public ReadOnly Property SelectedClockSourceIndex As UInteger
        Get
            Return _selectedClockSourceIndex
        End Get
    End Property

    Private _selectedCounterModeIndex As UInteger
    Public ReadOnly Property SelectedCounterModeIndex As UInteger
        Get
            Return _selectedCounterModeIndex
        End Get
    End Property

    Private _selectedStartValue As UInteger
    Public ReadOnly Property SelectedStartValue As UInteger
        Get
            Return _selectedStartValue
        End Get
    End Property

    Function ConfigureModule(clockSourceIndex As Integer, counterModeIndex As Integer, startValue As UInteger) As UInteger
        Dim clockSource As UInteger = ClockSourceValues(clockSourceIndex).ClockSourceValue
        Dim counterMode As UInteger = AdqCommon.Counter8254Modes(counterModeIndex)

        Dim adqError As UInteger = AdqDriver.adqCNTRConfig(BoardIndex, ModuleIndex, clockSource, counterMode, startValue)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            ErrorTextChanged("adqCNTRConfig", adqError)
        ElseIf adqError = AdqErrors.ADQ_ERROR_NONE Then
            _selectedClockSourceIndex = CUInt(clockSourceIndex)
            _selectedCounterModeIndex = CUInt(counterModeIndex)
            _selectedStartValue = startValue
        End If
        Return adqError
    End Function
End Class
