﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the WriteBufferAO project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Public Class MorletWaveletGenerator
    Implements ISignalGenerator

    Private Const _constant As Double = 0.95

    Public ReadOnly Property GeneratorIcon As String Implements ISignalGenerator.GeneratorIcon
        Get
            Return "/WriteBufferAO;component/Images/signalMorletWavelet.png"
        End Get
    End Property

    Public ReadOnly Property GeneratorName As String Implements ISignalGenerator.GeneratorName
        Get
            Return "Morlet wavelet"
        End Get
    End Property

    Public Function GetValue(cycleTime As Double, signalMinimum As Double, signalMaximum As Double) As Double Implements ISignalGenerator.GetValue
        ' https://en.wikipedia.org/wiki/Morlet_wavelet
        ' Morlet wavelet real part
        Dim x As Double = cycleTime * 6.0 - 3.0
        Dim xSquared As Double = x * x
        Return (signalMaximum + signalMinimum) / 2.0 + (signalMaximum - signalMinimum) / 2.0 * Math.Pow(Math.E, -xSquared / 2.0) * Math.Cos(AdqCommon.TwoPI * _constant * x)

        ' Morlet wavelet imaginary part
        'Return (signalMaximum + signalMinimum) / 2.0 + (signalMaximum - signalMinimum) / 2.0 * Math.Pow(Math.E, -xSquared / 2.0) * Math.Sin(AdqCommon.TwoPI * _constant * x)
    End Function
End Class
