﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the WriteBufferAO project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Public Class ShannonWaveletGenerator
    Implements ISignalGenerator

    Private Const _piHalf As Double = Math.PI / 2.0

    Public ReadOnly Property GeneratorIcon As String Implements ISignalGenerator.GeneratorIcon
        Get
            Return "/WriteBufferAO;component/Images/signalShannonWavelet.png"
        End Get
    End Property

    Public ReadOnly Property GeneratorName As String Implements ISignalGenerator.GeneratorName
        Get
            Return "Shannon wavelet"
        End Get
    End Property

    Public Function GetValue(cycleTime As Double, signalMinimum As Double, signalMaximum As Double) As Double Implements ISignalGenerator.GetValue
        ' https://en.wikipedia.org/wiki/Shannon_wavelet
        Dim x As Double = cycleTime * 8 - 4.0
        Dim k As Double = _piHalf * x
        Dim val As Double
        If k = 0 Then
            val = (signalMaximum + signalMinimum) / 2.0 + (signalMaximum - signalMinimum) / 2.0
        Else
            val = (signalMaximum + signalMinimum) / 2.0 + (signalMaximum - signalMinimum) / 2.0 * (Math.Sin(k) / k) * Math.Cos(3 * k)
        End If
        Return val
    End Function
End Class
