﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the WriteBufferAO project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Public Class InputValidation
    Public Shared Function ValidateValueIsWithinRange(textValue As String, minLimit As Double, maxLimit As Double, ByRef doubleValue As Double) As String
        doubleValue = 0.0
        Dim value As Double
        Dim msg As String = ValidateDoubleValue(textValue, value)
        If String.IsNullOrEmpty(msg) Then
            If (value < minLimit) OrElse (value > maxLimit) Then
                msg = String.Format("The entered value {2} is outside the limits ({0}...{1})", minLimit, maxLimit, value)
            Else
                doubleValue = value
            End If
        End If
        Return msg
    End Function

    Public Shared Function ValidateValueIsPositive(textValue As String, ByRef doubleValue As Double) As String
        doubleValue = 0.0
        Dim value As Double
        Dim msg As String = ValidateDoubleValue(textValue, value)
        If String.IsNullOrEmpty(msg) Then
            If value <= 0 Then
                msg = "The value must be a positive number"
            Else
                doubleValue = value
            End If
        End If
        Return msg
    End Function

    Private Shared Function ValidateDoubleValue(textValue As String, ByRef doubleValue As Double) As String
        doubleValue = 0.0
        Dim msg As String = Nothing
        If String.IsNullOrEmpty(textValue) Then
            msg = "The value cannot be empty"
        ElseIf Not Double.TryParse(textValue, doubleValue) Then
            msg = "The entered text """ + textValue + """ is not a valid number"
        End If
        Return msg
    End Function
End Class
