//-----------------------------------------------------------------------------
// adqDeviceErrors.h - Error constants for ALLDAQ driver system API
//      Copyright (c) ALLDAQ, a division of ALLNET GmbH
//      https://www.alldaq.com
//
// Purpose:
//      This file has defines for error constants
//      used by ALLDAQ measurement boards
//-----------------------------------------------------------------------------

#ifndef ADQDEVICEERRORS_H
#define ADQDEVICEERRORS_H

/*! \name	Constants for error codes - The return value from all API functions
       
*/

//@{

//! Bits 28 to 31 contain the error type

const unsigned int ADQ_ERROR_TYPE_MASK              = 0xF0000000;   //!<    Mask for the error type - Bits 28 to 31 of the error code. 

const unsigned int ADQ_ERROR_TYPE_FATAL             = 0x80000000;   //!<    Fatal error - The driver system cannot (or no longer) be used.

const unsigned int ADQ_ERROR_TYPE_ERROR             = 0x40000000;   //!<    Standard error - The operation was unsuccessful, but the driver system is not affected.

const unsigned int ADQ_ERROR_TYPE_WARNING           = 0x20000000;   //!<    Warning - The operation was only partially successful.

const unsigned int ADQ_ERROR_TYPE_HINT              = 0x10000000;   //!<    Hint - For more detailed information adqGetExtendedError should be called.

const unsigned int ADQ_ERROR_TYPE_SUCCESS           = 0x00000000;   //!<    No error occured.


//! If Bit 25 is set, then internal information is available via adqGetInternalError

const unsigned int ADQ_ERROR_INTERNAL_INFORMATION   = 0x02000000;   //!<    Bit 25 - If this bit is set, then adqGetInternalError should be called and the returned information sent to ALLDAQ 

//! If Bit 24 is set, then more information is available via adqGetExtendedError

const unsigned int ADQ_ERROR_MORE_INFORMATION       = 0x01000000;   //!<    Bit 24 - If this bit is set, then adqGetExtendedError should be called for more detailed information 


//! Bits 16 to 19 contain the index of the parameter which has caused the error - 1 based.

const unsigned int ADQ_ERROR_PARAMETER_MASK                     = 0x000F0000;   //!<    Mask for the parameter - Bits 16 to 19 of the error code. 

const unsigned int ADQ_ERROR_PARAMETER_NONE                     = 0x00000000;   //!<    If bits 16 to 19 are all 0, then the error is general and not caused by a particular parameter. 

const unsigned int ADQ_ERROR_PARAMETER_1                        = 0x00010000;   //!<    The first parameter caused the error. 

const unsigned int ADQ_ERROR_PARAMETER_2                        = 0x00020000;   //!<    The second parameter caused the error. 

const unsigned int ADQ_ERROR_PARAMETER_3                        = 0x00030000;   //!<    The third parameter caused the error. 

const unsigned int ADQ_ERROR_PARAMETER_4                        = 0x00040000;   //!<    The fourth parameter caused the error. 

const unsigned int ADQ_ERROR_PARAMETER_5                        = 0x00050000;   //!<    The fifth parameter caused the error. 

const unsigned int ADQ_ERROR_PARAMETER_6                        = 0x00060000;   //!<    The sixth parameter caused the error. 

const unsigned int ADQ_ERROR_PARAMETER_7                        = 0x00070000;   //!<    The seventh parameter caused the error. 

const unsigned int ADQ_ERROR_PARAMETER_8                        = 0x00080000;   //!<    The eighth parameter caused the error. 

const unsigned int ADQ_ERROR_PARAMETER_9                        = 0x00090000;   //!<    The ninth parameter caused the error. 

const unsigned int ADQ_ERROR_PARAMETER_10                       = 0x000A0000;   //!<    The tenth parameter caused the error. 

const unsigned int ADQ_ERROR_PARAMETER_11                       = 0x000B0000;   //!<    The eleventh parameter caused the error. 


//! The error codes themselves are in the lower 16 bits.

const unsigned int ADQ_ERROR_NONE                               = 0x00000000;   //!<    No error

const unsigned int ADQ_ERROR_INTERNAL                           = 0x00000001;   //!<    Internal error - If the bit ADQ_ERROR_MORE_INFORMATION (see above) is set, then call adqGetExtendedError

const unsigned int ADQ_ERROR_INVALID_PARAMETER                  = 0x00000002;   //!<    Invalid parameter - The number of the parameter (starting with 1) which caused the error is in bits 16 to 19 (see above)

const unsigned int ADQ_ERROR_NO_DEVICES                         = 0x00000003;   //!<    Not a single ADQ device was found.

const unsigned int ADQ_ERROR_ALREADY_OPEN                       = 0x00000004;   //!<    adqOpen or adqOpenEx was called more than once

const unsigned int ADQ_ERROR_NOT_OPEN                           = 0x00000005;   //!<    adqClose was called although neither adqOpen or adqOpenEx had been called

const unsigned int ADQ_ERROR_INVALID_BOARD_INDEX                = 0x00000006;   //!<    On calling adqQuery with 'ADQ_QUERY_SYSTEM_BOARD_HANDLE' a  board index was used which did not exist.

const unsigned int ADQ_ERROR_CANNOT_OPEN_BOARD                  = 0x00000007;   //!<    The internal function to open the device driver failed.

const unsigned int ADQ_ERROR_INVALID_MODULE_INDEX               = 0x00000008;   //!<    The board does not have a module with this index or ADQ_SYSTEM_HANDLE was used without ADQ_INDEX_MODULE_ALL in adqQuery

const unsigned int ADQ_ERROR_UNKNOWN_SYSTEM_QUERY               = 0x00000009;   //!<    The ID passed to the function adqQuery does not exist.

const unsigned int ADQ_ERROR_BUFFER_TOO_SMALL                   = 0x0000000A;   //!<    The size of the buffer is to small to hold the data.

const unsigned int ADQ_ERROR_UNSUPPORTED                        = 0x0000000B;   //!<    The function is not supported by this module.
                                                                                        
const unsigned int ADQ_ERROR_IRQ_ALREADY_ENABLED                = 0x0000000C;   //!<    adqEnableIRQ was called, but the IRQ is already enabled.
                                                                                        
const unsigned int ADQ_ERROR_IRQ_NOT_ENABLED                    = 0x0000000D;   //!<    adqDisableIRQ was called, but the IRQ is not enabled.

const unsigned int ADQ_ERROR_TIMEOUT                            = 0x0000000E;   //!<    The function did not respond within the given timeout value. 

const unsigned int ADQ_ERROR_MODULE_BUSY                        = 0x0000000F;   //!<    The module is performing an operation, e.g. a timer controlled aquisition or output.

const unsigned int ADQ_ERROR_KERNEL_BUFFER_OVERFLOW             = 0x00000010;   //!<    The buffer in kernel space cannot store any more data.

const unsigned int ADQ_ERROR_AI_INVALID_CHANNEL_INDEX           = 0x00000011;   //!<    The AI module does not have a channel with this index.

const unsigned int ADQ_ERROR_AI_INVALID_RANGE_INDEX             = 0x00000012;   //!<    The AI channel does not have a range with this index.

const unsigned int ADQ_ERROR_AI_INVALID_FRAME_LENGTH            = 0x00000013;   //!<    The given frame length does not correspond to the channel list passed in the last call to function adqAISetChannelList. 

const unsigned int ADQ_ERROR_WRITE_OP_NOT_RUNNING               = 0x00000014;   //!<    adqWriteContinuous has been called, but a write operation is not currently running

const unsigned int ADQ_ERROR_KERNEL_BUFFER_UNDERFLOW            = 0x00000015;   //!<    The buffer in kernel space is empty.

const unsigned int ADQ_ERROR_STRUCTURE_SIZE_OR_ID_FALSE         = 0x00000016;   //!<    The size or ID of a structure passed to the routine was incorrect

const unsigned int ADQ_ERROR_STRUCTURE_ID_FALSE                 = 0x00000017;   //!<    The structure ID passed to a routine was incorrect

const unsigned int ADQ_ERROR_NETWORK_COMMUNICATION              = 0x00000018;   //!<    A general network error occurred on calling a routine on a remote system.

const unsigned int ADQ_ERROR_NETWORK_TIMEOUT                    = 0x00000019;   //!<    A remote system did not reply within a reasonable period of time.

const unsigned int ADQ_ERROR_FREQUENCY_INVALID                  = 0x0000001A;   //!<    A valid frequency was not set for this module before starting a continuous operation.

const unsigned int ADQ_ERROR_SYNCHRON_START_INVALID_GROUP_INDEX = 0x0000001B;   //!<    The synchronous start group index is not valid for this board.

const unsigned int ADQ_ERROR_SYNCHRON_START_INVALID_MODULES     = 0x0000001C;   //!<    One or more of the modules in the module map for a synchronous start does not support a synchronous start

const unsigned int ADQ_ERROR_SYNCHRON_START_TOO_MANY_MODULES    = 0x0000001D;   //!<    Too many modules have been included in a group for a synchronous start

//@}

#endif