#include <windows.h>

#include <stdio.h>
#include <stdlib.h>

#include <tchar.h>

#include <conio.h>

#include "..\\adqDevice.h"

#ifdef WIN64

#pragma comment(lib, "..\\adqDevice_64.lib")

#else

#pragma comment(lib, "..\\adqDevice.lib")

#endif

int _tmain(int argc, _TCHAR* argv[])
{
    // Increase the size of the console buffer
    
    HANDLE  h_console = GetStdHandle(STD_OUTPUT_HANDLE);

    COORD coord;

    coord.X = 200;
    
    coord.Y = 9999;
    
    SetConsoleScreenBufferSize(h_console, coord);

    bool b_terminate = false;

    bool b_remote = false;

    wchar_t wsz_remote_name_or_IP_address[128];

    while(true)
    {
        printf( "1. Open local\n"
                "2. Open remote\n"
                "\n"
                "Your choice: 1 or 2 (RETURN to exit): ");

        char sz_response[8];

        gets_s(&sz_response[0], 8);

        printf("\n\n");

        if(sz_response[0] == 0)
        {
            b_terminate = true;
            
            break;
        }

        unsigned int ui_choice = 0;

        if( sscanf_s(&sz_response[0], "%u", &ui_choice) != 1)
        {
            continue;
        }

        if(ui_choice == 1)
        {
            break;
        }

        if(ui_choice == 2)
        {
            b_remote = true;

            printf("Name or IP-Address (RETURN to exit): ");

            _getws_s(&wsz_remote_name_or_IP_address[0], 128);

            printf("\n\n");

            if( wsz_remote_name_or_IP_address[0] == (wchar_t)0 )
            {
                b_terminate = true;
            }

            break;
        }
    }

    if(b_terminate)
    {
        printf("Press any key to exit.\n");

        _getch();

        exit(-1);
    }

    unsigned int ui_adq_error;
    
    if(b_remote)
    {
        wchar_t* pwc_user_name_or_address[2] = {&wsz_remote_name_or_IP_address[0],
                                                NULL};

        ui_adq_error = adqOpenEx(pwc_user_name_or_address);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("adqOpenEx returned 0x%08X\n\n", ui_adq_error);

            printf("Press any key to exit.\n");

            _getch();

            exit(-1);
        }
    }
    else
    {
        ui_adq_error = adqOpen();

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("adqOpen returned 0x%08X\n\n", ui_adq_error);

            printf("Press any key to exit.\n");

            _getch();

            exit(-1);
        }
    }

    SADQSystemInfo s_adq_system_info;

    memset( &s_adq_system_info, 0, sizeof(s_adq_system_info) );

    s_adq_system_info.m_uiSize = sizeof(SADQSystemInfo);

    s_adq_system_info.m_uiID = SADQSystemInfo_ID;

    ui_adq_error = adqGetSystemInfo(s_adq_system_info);

    if(ui_adq_error != ADQ_ERROR_NONE)
    {
        printf("adqGetSystemInfo returned 0x%08X\n\n", ui_adq_error);

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }


    printf("ALLDAQ DriverSystem\n");
    printf("    DLL version:                %u.%u.%u.%u\n"
           "    Boards found:               %u\n", 
           (s_adq_system_info.m_uiDLLVersion & 0xFF000000) >> 24, (s_adq_system_info.m_uiDLLVersion & 0x00FF0000) >> 16, (s_adq_system_info.m_uiDLLVersion & 0x0000FF00) >> 8, s_adq_system_info.m_uiDLLVersion & 0x000000FF,
           s_adq_system_info.m_uiNumberOfBoards);

    if(s_adq_system_info.m_uiNumberOfBoards == 0)
    {
        printf("NO ALLDAQ boards found.\n\n");

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(0);
    }

    for(unsigned int ui_index_board = 0; ui_index_board < s_adq_system_info.m_uiNumberOfBoards; ui_index_board++)
    {
        printf("\n\n");

        printf("Board %3d\n"
               "---------\n\n",
               ui_index_board   );

        SADQBoardInfo s_adq_board_info;

        s_adq_board_info.m_uiSize = sizeof(SADQBoardInfo);

        s_adq_board_info.m_uiID = SADQBoardInfo_ID;

        ui_adq_error = adqGetBoardInfo(ui_index_board, s_adq_board_info);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("    Board index: %3u - adqGetBoardInfo returned 0x%08X\n\n", ui_index_board, ui_adq_error);

            printf("Press any key to continue.\n");

            _getch();

            continue;
        }

        wchar_t* wsz_bus_type = L"????";

        if (s_adq_board_info.m_uiBusType == ADQ_BUS_TYPE_PCI)
        {
            wsz_bus_type = L"PCI";
        }
        else if (s_adq_board_info.m_uiBusType == ADQ_BUS_TYPE_USB)
        {
            wsz_bus_type = L"USB";
        }

        printf("    Vendor ID:                  0x%04X\n"
               "    Device ID:                  0x%04X\n"
               "    Serial number:              0x%08X\n"
               "    Board name:                 %S\n"
               "    Board description:          %S\n"
               "    Driver name:                %S\n"
               "    Driver version:             %u.%u.%u.%u\n"
               "    Firmware version:           %u\n"
               "    Bus type:                   %S (0x%08X)\n"
               "    Remote:                     %S\n"
               "    Remote address:             %S\n",
               s_adq_board_info.m_usVendorID,
               s_adq_board_info.m_usDeviceID,
               s_adq_board_info.m_uiSerialNumber,
               &s_adq_board_info.m_wszBoardName[0],
               &s_adq_board_info.m_wszBoardDescription[0],
               &s_adq_board_info.m_wszDriverName[0],
               (s_adq_board_info.m_uiDriverVersion & 0xFF000000) >> 24, (s_adq_board_info.m_uiDriverVersion & 0x00FF0000) >> 16,  (s_adq_board_info.m_uiDriverVersion & 0x0000FF00) >> 8, s_adq_board_info.m_uiDriverVersion & 0x000000FF,
               s_adq_board_info.m_uiFirmwareVersion,
               wsz_bus_type,
               s_adq_board_info.m_uiBusType,
               (s_adq_board_info.m_bRemote ? L"Yes" : L"No"),
               &s_adq_board_info.m_wszRemoteIPAddressOrHostName[0]);

            
        printf("    TTL voltage levels:         0x%08X\n", 
               s_adq_board_info.m_uiTTLVoltageLevel);
            
        if ((s_adq_board_info.m_uiTTLVoltageLevel & ADQ_TTL_VOLTAGE_LEVEL_5_0_VOLTS) == ADQ_TTL_VOLTAGE_LEVEL_5_0_VOLTS)
        {
            printf("        5.0 V                   0x%08X\n",
                  ADQ_TTL_VOLTAGE_LEVEL_5_0_VOLTS);
        }

        if ((s_adq_board_info.m_uiTTLVoltageLevel & ADQ_TTL_VOLTAGE_LEVEL_3_3_VOLTS) == ADQ_TTL_VOLTAGE_LEVEL_3_3_VOLTS)
        {
            printf("        3.3 V                   0x%08X\n",
                  ADQ_TTL_VOLTAGE_LEVEL_3_3_VOLTS);
        }
            
        if (s_adq_board_info.m_uiTTLVoltageLevel > 0)
        {
            unsigned int ui_voltage_level;

            ui_adq_error = adqGetBoardTTLVoltage(ui_index_board, ui_voltage_level);

            if (ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("    Board index: %3u - adqGetBoardTTLVoltage returned 0x%08X\n\n", ui_index_board, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            wchar_t* wsz_voltage_level = L"????";
            
            if (ui_voltage_level == ADQ_TTL_VOLTAGE_LEVEL_5_0_VOLTS)
            {
                wsz_voltage_level = L"5.0 V";
            }
            else if (ui_voltage_level == ADQ_TTL_VOLTAGE_LEVEL_3_3_VOLTS)
            {
                wsz_voltage_level = L"3.3 V";
            }

            printf("    Current TTL voltage level:  %S (0x%08X)\n", 
                   wsz_voltage_level,
                   ui_voltage_level);
        }

        printf("\n");


        if(s_adq_board_info.m_uiBusType == ADQ_BUS_TYPE_PCI)
        {
            SADQBoardPCIInfo s_adq_board_pci_info;

            s_adq_board_pci_info.m_uiSize = sizeof(SADQBoardPCIInfo);

            s_adq_board_pci_info.m_uiID = SADQBoardPCIInfo_ID;

            ui_adq_error = adqGetBoardPCIInfo(ui_index_board, s_adq_board_pci_info);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("    Board index: %3u - adqGetBoardPCIInfo returned 0x%08X\n\n", ui_index_board, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            printf("    PCI Location info:\n"
                   "        PCI-bus number:         %u\n"
                   "        PCI device number:      %u\n"
                   "        PCI function number:    %u\n\n",
                   s_adq_board_pci_info.m_uiBusNumber,
                   s_adq_board_pci_info.m_uiDeviceNumber,
                   s_adq_board_pci_info.m_uiFunctionNumber  );
        }

        printf("    %u Modules on board\n", s_adq_board_info.m_uiNumberOfModules);

        for(unsigned int ui_index_module = 0; ui_index_module < s_adq_board_info.m_uiNumberOfModules; ui_index_module++)
        {
            printf("\n\n");

            printf("    Module %3d\n"
                   "    ----------\n\n",
                   ui_index_module      );

            unsigned int ui_module_type;

            ui_adq_error = adqGetModuleType(ui_index_board, ui_index_module, ui_module_type);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("        Board index: %3u Module index: %3u - adqGetModuleType returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            SADQGeneralModuleInfo s_adq_general_module_info;

            s_adq_general_module_info.m_uiSize = sizeof(SADQGeneralModuleInfo);

            s_adq_general_module_info.m_uiID = SADQGeneralModuleInfo_ID;

            unsigned int ui_info_size;

            ui_adq_error = adqGetModuleInfo(ui_index_board, ui_index_module, s_adq_general_module_info, 0, NULL, ui_info_size);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("        Board index: %3u Module index: %3u adqGetModuleInfo returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            wchar_t* wsz_module_type = L"????";

            switch (ui_module_type)
            {
                case ADQ_MODULE_TYPE_DI:
                {
                    wsz_module_type = L"DI";

                    break;
                }
                case ADQ_MODULE_TYPE_DO:
                {
                    wsz_module_type = L"DO";

                    break;
                }
                case ADQ_MODULE_TYPE_DIO:
                {
                    wsz_module_type = L"DIO";

                    break;
                }
                case ADQ_MODULE_TYPE_CNTR:
                {
                    wsz_module_type = L"Counter";

                    break;
                }
                case ADQ_MODULE_TYPE_EXTINT:
                {
                    wsz_module_type = L"External Interrupt";

                    break;
                }
                case ADQ_MODULE_TYPE_AI:
                {
                    wsz_module_type = L"Analog Input";

                    break;
                }
                case ADQ_MODULE_TYPE_AO:
                {
                    wsz_module_type = L"Analog Output";

                    break;
                }
                case ADQ_MODULE_TYPE_I2C:
                {
                    wsz_module_type = L"I2C";

                    break;
                }
                case ADQ_MODULE_TYPE_PWM:
                {
                    wsz_module_type = L"PWM";

                    break;
                }
                case ADQ_MODULE_TYPE_FIN:
                {
                    wsz_module_type = L"FIN";

                    break;
                }
                case ADQ_MODULE_TYPE_I2SO:
                {
                    wsz_module_type = L"I2S output";

                    break;
                }
            }

            wchar_t* wsz_clock_type = L"????";

            if(s_adq_general_module_info.m_uiClockType == ADQ_CLOCK_TYPE_NONE)
            {
                wsz_clock_type = L"None";
            }
            else if(s_adq_general_module_info.m_uiClockType == ADQ_CLOCK_TYPE_SOFTWARE)
            {
                wsz_clock_type = L"Software";
            }
            else if(s_adq_general_module_info.m_uiClockType == ADQ_CLOCK_TYPE_HARDWARE)
            {
                wsz_clock_type = L"Hardware";
            }

            double d_max_sampling_frequency = 0.0;

            if(s_adq_general_module_info.m_uiClockType != ADQ_CLOCK_TYPE_NONE)
            {
                ui_adq_error = adqGetMaximumSampleFrequency(ui_index_board, ui_index_module, 1, d_max_sampling_frequency);

                if(ui_adq_error != ADQ_ERROR_NONE)
                {
                    printf("        Board index: %3u Module index: %3u adqGetMaximumSampleFrequency returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                    printf("Press any key to continue.\n");

                    _getch();

                    continue;
                }
            }

            printf( "        Module description:               %S\n\n"
                    "        Module type:                      %S (0x%08X)\n\n"
                    "        IRQ Source:                       %S\n\n"
                    "        Clock type:                       %S (0x%08X)\n",
                    &s_adq_general_module_info.m_wszModuleDescription[0],
                    wsz_module_type,
                    ui_module_type,
                    (s_adq_general_module_info.m_bIsIRQSource ? L"Yes" : L"No"),
                    wsz_clock_type,
                    s_adq_general_module_info.m_uiClockType);

            if(s_adq_general_module_info.m_uiClockType != ADQ_CLOCK_TYPE_NONE)
            {
                printf( "        Clock base frequency:             %f Hz\n"
                        "        Clock minimum divider:            %u\n"
                        "        Maximum sample frequency:         %f Hz\n\n"
                        "        Internal data buffer size:        %u values\n",
                        s_adq_general_module_info.m_dClockBaseFrequencyHz,
                        s_adq_general_module_info.m_uiClockMinimumDivider,
                        d_max_sampling_frequency,
                        s_adq_general_module_info.m_uiInternalDataBufferSizeValues);
            }

            printf("\n");
            
            SADQHWTriggerInfo s_adq_hardware_trigger_info;

            memset( &s_adq_hardware_trigger_info, 0, sizeof(SADQHWTriggerInfo) );

            s_adq_hardware_trigger_info.m_uiSize = sizeof(SADQHWTriggerInfo);

            s_adq_hardware_trigger_info.m_uiID = SADQHWTriggerInfo_ID;

            ui_adq_error = adqGetHWTriggerInfo(ui_index_board, ui_index_module, s_adq_hardware_trigger_info);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("        Board index: %3u Module index: %3u adqGetHWTriggerInfo returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            if( (s_adq_hardware_trigger_info.m_uiHWTriggerStartSource == 0x00000000)&&
                                (s_adq_hardware_trigger_info.m_uiHWTriggerStopSource == 0x00000000) )
            {
                printf( "        Hardware trigger:                 None\n\n");
            }
            else
            {
                printf( "        Hardware trigger:\n");

                if(s_adq_hardware_trigger_info.m_uiHWTriggerStartSource != 0x00000000)
                {
                    printf( "            Start trigger:                ");

                    unsigned int ui_mask = 0x00000001;

                    char c_id = 'A';

                    int b_first_trigger = 1;

                    for(unsigned int bit_id = 0; bit_id < 32; bit_id++)
                    {
                        if( (s_adq_hardware_trigger_info.m_uiHWTriggerStartSource & ui_mask) == ui_mask )
                        {
                            if(b_first_trigger == 0)
                            {
                                printf(", ");
                            }
                            
                            printf("%c", c_id);

                            b_first_trigger = 0;
                        }

                        ui_mask<<= 1;

                        c_id+= 1;
                    }

                    printf("\n");
                }

                if(s_adq_hardware_trigger_info.m_uiHWTriggerStopSource != 0x00000000)
                {
                    printf( "            Stop trigger:                 ");

                    unsigned int ui_mask = 0x00000001;

                    char c_id = 'A';

                    int b_first_trigger = 1;

                    for(unsigned int bit_id = 0; bit_id < 32; bit_id++)
                    {
                        if( (s_adq_hardware_trigger_info.m_uiHWTriggerStopSource & ui_mask) == ui_mask )
                        {
                            if(b_first_trigger == 0)
                            {
                                printf(", ");
                            }
                            
                            printf("%c", c_id);

                            b_first_trigger = 0;
                        }

                        ui_mask <<= 1;

                        c_id++;
                    }

                    printf("\n");
                }

                
                printf("            Channel list per trigger:     %S\n",
                    s_adq_hardware_trigger_info.m_bSupportsChannelListPerTrigger ? L"Yes" : L"No");

                printf("\n");
            }

            if (s_adq_general_module_info.m_bIsIRQSource)
            {

                SADQIRQInfo s_adq_irq_info;
            
                memset(&s_adq_irq_info, 0, sizeof(SADQIRQInfo));

                s_adq_irq_info.m_uiSize = sizeof(SADQIRQInfo);

                s_adq_irq_info.m_uiID = SADQIRQInfo_ID;
            
                ui_adq_error = adqGetIRQInfo(ui_index_board, ui_index_module, s_adq_irq_info);

                if (ui_adq_error != ADQ_ERROR_NONE)
                {
                    printf("        Board index: %3u Module index: %3u adqGetIRQInfo returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                    printf("Press any key to continue.\n");

                    _getch();

                    continue;
                }

                printf("        Interrupt info:\n"
                       "            Interrupt description:        %S\n"
                       "            Supports bit-change:          %S\n"
                       "            Supports bit-pattern compare: %S\n"
                       "            External IRQ:                 0x%08X\n"
                       "            Port status error:            0x%08X\n"
                       "            Fault recovery:               0x%08X\n\n",
                       s_adq_irq_info.m_wszIRQDescription,
                       s_adq_irq_info.m_bSupportsBitChange ? L"Yes" : L"No",
                       s_adq_irq_info.m_bSupportsBitPatternCompare ? L"Yes" : L"No",
                       s_adq_irq_info.m_uiSupportsExternal,
                       s_adq_irq_info.m_uiSupportsPortStatusError,
                       s_adq_irq_info.m_uiSupportsFaultRecovery);
            }

            switch(ui_module_type)
            {
                case ADQ_MODULE_TYPE_DI:
                {
                    SADQDIModuleInfo s_adq_di_module_info;

                    s_adq_di_module_info.m_uiSize = sizeof(SADQDIModuleInfo);

                    s_adq_di_module_info.m_uiID = SADQDIModuleInfo_ID;

                    ui_adq_error = adqGetModuleInfo(ui_index_board, ui_index_module, s_adq_general_module_info, sizeof(SADQDIModuleInfo), (unsigned char*)&s_adq_di_module_info, ui_info_size);

                    if (ui_adq_error != ADQ_ERROR_NONE)
                    {
                        printf("        Board index: %3u Module index: %3u adqGetModuleInfo returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                        printf("Press any key to continue.\n");

                        _getch();

                        continue;
                    }

                    printf("        Port width:                       %u bits\n"
                           "        Opto-isolated:                    %S\n"
                           "        TTL:                              %S\n"
                           "        Number of filters:                %u\n",
                           s_adq_di_module_info.m_uiPortWidthBits,
                           s_adq_di_module_info.m_bIsOptoIsolated ? L"Yes" : L"No",
                           s_adq_di_module_info.m_bIsTTL ? L"Yes" : L"No",
                           s_adq_di_module_info.m_uiNumberOfFilters);

                    for (unsigned int index_filter = 0; index_filter < s_adq_di_module_info.m_uiNumberOfFilters; index_filter++)
                    {
                        printf("            Filter[%u]:                    %I64u psec\n",
                                index_filter,
                                s_adq_di_module_info.m_arrui64FilterTimePicoseconds[index_filter]);
                    }

                    if (s_adq_di_module_info.m_uiNumberOfFilters > 0)
                    {
                        unsigned long long ull_filter_picoseconds;
                        
                        ui_adq_error = adqDIOGetFilterPicoseconds(ui_index_board, ui_index_module, ull_filter_picoseconds);
                        
                        if (ui_adq_error != ADQ_ERROR_NONE)
                        {
                            printf("        Board index: %3u Module index: %3u adqDIOGetFilterPicoseconds returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                            printf("Press any key to continue.\n");

                            _getch();

                            continue;
                        }

                        printf("        Current filter:                   %I64u psec\n",
                            ull_filter_picoseconds);
                    }

                    printf("\n");

                    break;
                }
                case ADQ_MODULE_TYPE_DO:
                {
                    SADQDOModuleInfo s_adq_do_module_info;

                    s_adq_do_module_info.m_uiSize = sizeof(SADQDOModuleInfo);

                    s_adq_do_module_info.m_uiID = SADQDOModuleInfo_ID;

                    ui_adq_error = adqGetModuleInfo(ui_index_board, ui_index_module, s_adq_general_module_info, sizeof(SADQDOModuleInfo), (unsigned char*)&s_adq_do_module_info, ui_info_size);

                    if (ui_adq_error != ADQ_ERROR_NONE)
                    {
                        printf("        Board index: %3u Module index: %3u adqGetModuleInfo returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                        printf("Press any key to continue.\n");

                        _getch();

                        continue;
                    }

                    printf("        Port width:                       %u bits\n"
                           "        Opto-isolated:                    %S\n"
                           "        TTL:                              %S\n\n",
                           s_adq_do_module_info.m_uiPortWidthBits,
                           s_adq_do_module_info.m_bIsOptoIsolated ? L"Yes" : L"No",
                           s_adq_do_module_info.m_bIsTTL ? L"Yes" : L"No");

                    break;
                }
                case ADQ_MODULE_TYPE_DIO:
                {
                    SADQDIOModuleInfo s_adq_dio_module_info;

                    s_adq_dio_module_info.m_uiSize = sizeof(SADQDIOModuleInfo);

                    s_adq_dio_module_info.m_uiID = SADQDIOModuleInfo_ID;

                    ui_adq_error = adqGetModuleInfo(ui_index_board, ui_index_module, s_adq_general_module_info, sizeof(SADQDIOModuleInfo), (unsigned char*)&s_adq_dio_module_info, ui_info_size);

                    if (ui_adq_error != ADQ_ERROR_NONE)
                    {
                        printf("        Board index: %3u Module index: %3u adqGetModuleInfo returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                        printf("Press any key to continue.\n");

                        _getch();

                        continue;
                    }

                    printf("        Port width:                       %u bits\n"
                           "        Opto-isolated:                    %S\n"
                           "        TTL:                              %S\n",
                           s_adq_dio_module_info.m_uiPortWidthBits,
                           s_adq_dio_module_info.m_bIsOptoIsolated ? L"Yes" : L"No",
                           s_adq_dio_module_info.m_bIsTTL ? L"Yes" : L"No");

                    unsigned int ui_direction;

                    ui_adq_error = adqDIOGetDirection(ui_index_board, ui_index_module, ui_direction);

                    wchar_t* wsz_direction = L"????";

                    if (ui_direction == ADQ_DIO_INPUT)
                    {
                        wsz_direction = L"Input";
                    }
                    else if (ui_direction == ADQ_DIO_OUTPUT)
                    {
                        wsz_direction = L"Output";
                    }

                    printf("        Current DIO direction:            %S (0x%08X)\n\n",
                           wsz_direction, 
                           ui_direction);

                    break;
                }
                case ADQ_MODULE_TYPE_CNTR:
                {
                    SADQCNTRModuleInfo s_adq_cntr_module_info;

                    s_adq_cntr_module_info.m_uiSize = sizeof(SADQCNTRModuleInfo);

                    s_adq_cntr_module_info.m_uiID = SADQCNTRModuleInfo_ID;

                    ui_adq_error = adqGetModuleInfo(ui_index_board, ui_index_module, s_adq_general_module_info, sizeof(SADQCNTRModuleInfo), (unsigned char*)&s_adq_cntr_module_info, ui_info_size);

                    if (ui_adq_error != ADQ_ERROR_NONE)
                    {
                        printf("        Board index: %3u Module index: %3u adqGetModuleInfo returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                        printf("Press any key to continue.\n");

                        _getch();

                        continue;
                    }

                    wchar_t* wsz_counter_type = L"????";

                    if (s_adq_cntr_module_info.m_uiType == ADQ_CNTR_TYPE_8254)
                    {
                        wsz_counter_type = L"8254";
                    }
                    else if (s_adq_cntr_module_info.m_uiType == ADQ_CNTR_TYPE_COUNT_DOWN_32_BIT)
                    {
                        wsz_counter_type = L"Count Down 32-Bit";
                    }

                    long long ll_max_value = (1LL << s_adq_cntr_module_info.m_uiWidthBits) - 1LL;

                    printf("        Type:                             %S (0x%08X)\n"
                           "        Width:                            %u bits\n"
                           "        Maximum start value               %I64d (0x%llX)\n",
                           wsz_counter_type,
                           s_adq_cntr_module_info.m_uiType,
                           s_adq_cntr_module_info.m_uiWidthBits,
                           ll_max_value,
                           ll_max_value);

                    printf("        Clock sources:                    (0x%08X)\n",
                           s_adq_cntr_module_info.m_uiClockSources);

                    if ((s_adq_cntr_module_info.m_uiClockSources & ADQ_CNTR_CLOCK_SOURCE_EXTERNAL) == ADQ_CNTR_CLOCK_SOURCE_EXTERNAL)
                    {
                        printf("            External                      (0x%08X)\n", ADQ_CNTR_CLOCK_SOURCE_EXTERNAL);
                    }
                    
                    if ((s_adq_cntr_module_info.m_uiClockSources & ADQ_CNTR_CLOCK_SOURCE_INTERNAL_10_MHZ) == ADQ_CNTR_CLOCK_SOURCE_INTERNAL_10_MHZ)
                    {
                        printf("            Internal 10 MHz               (0x%08X)\n", ADQ_CNTR_CLOCK_SOURCE_INTERNAL_10_MHZ);
                    }
                    
                    if ((s_adq_cntr_module_info.m_uiClockSources & ADQ_CNTR_CLOCK_SOURCE_INTERNAL_66_MHZ) == ADQ_CNTR_CLOCK_SOURCE_INTERNAL_66_MHZ)
                    {
                        printf("            Internal 66 MHz               (0x%08X)\n", ADQ_CNTR_CLOCK_SOURCE_INTERNAL_66_MHZ);
                    }

                    break;
                }
                case ADQ_MODULE_TYPE_AI:
                {
                    SADQAIModuleInfo s_adq_ai_module_info;

                    s_adq_ai_module_info.m_uiSize = sizeof(SADQAIModuleInfo);

                    s_adq_ai_module_info.m_uiID = SADQAIModuleInfo_ID;

                    ui_adq_error = adqGetModuleInfo(ui_index_board, ui_index_module, s_adq_general_module_info, sizeof(SADQAIModuleInfo), (unsigned char*)&s_adq_ai_module_info, ui_info_size);

                    if (ui_adq_error != ADQ_ERROR_NONE)
                    {
                        printf("        Board index: %3u Module index: %3u adqGetModuleInfo returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                        printf("Press any key to continue.\n");

                        _getch();

                        continue;
                    }

                    if (s_adq_ai_module_info.m_bRangeInfoPerChannel)
                    {
                        printf("        Not all the channels have the same ranges - Program must be adapted\n\n");

                        printf("Press any key to continue.\n");

                        _getch();

                        continue;
                    }

                    unsigned int ui_selected_calibration;

                    ui_adq_error = adqAnalogGetSelectedCalibration(ui_index_board, ui_index_module, ui_selected_calibration);

                    if (ui_adq_error != ADQ_ERROR_NONE)
                    {
                        printf("        Board index: %3u Module index: %3u adqAnalogGetSelectedCalibration returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                        printf("Press any key to continue.\n");

                        _getch();

                        continue;
                    }

                    wchar_t* wsz_selected_calibration = L"????";

                    if (ui_selected_calibration == ADQ_CALIBRATION_FACTORY)
                    {
                        wsz_selected_calibration = L"Factory";
                    }
                    else if (ui_selected_calibration == ADQ_CALIBRATION_USER_1)
                    {
                        wsz_selected_calibration = L"User";
                    }

                    printf("        Shared data rate:                 %S\n"
                           "        Selected calibration:             %S (0x%08X)\n"
                           "        Number of channels:               %u\n",
                           s_adq_ai_module_info.m_bSharedDataRate ? L"Yes" : L"No",
                           wsz_selected_calibration,
                           ui_selected_calibration,
                           s_adq_ai_module_info.m_uiNumberOfChannels);

                    for (unsigned int ui_index_channel = 0; ui_index_channel < s_adq_ai_module_info.m_uiNumberOfChannels; ui_index_channel++)
                    {
                        unsigned int ui_index_range;
                        ui_adq_error = adqAIGetChannelHardwareRange(ui_index_board, ui_index_module, ui_index_channel, ui_index_range);

                        if (ui_adq_error != ADQ_ERROR_NONE)
                        {
                            if (ui_adq_error != (ADQ_ERROR_TYPE_ERROR | ADQ_ERROR_PARAMETER_2 | ADQ_ERROR_UNSUPPORTED))
                            {
                                printf("        Board index: %3u Module index: %3u adqAIGetChannelHardwareRange returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);
                            }

                            break;
                        }
                        else
                        {
                            printf("            Channel %u current range:      %u\n",
                                ui_index_channel,
                                ui_index_range);
                        }
                    }

                    printf("\n");

                    printf("        Maximum sample frequencies:\n");

                    for (unsigned int ui_number_of_channels = 1; ui_number_of_channels <= s_adq_ai_module_info.m_uiNumberOfChannels; ui_number_of_channels++)
                    {
                        ui_adq_error = adqGetMaximumSampleFrequency(ui_index_board, ui_index_module, ui_number_of_channels, d_max_sampling_frequency);

                        if (ui_adq_error != ADQ_ERROR_NONE)
                        {
                            printf("        Board index: %3u Module index: %3u adqGetMaximumSampleFrequency returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                            break;
                        }

                        printf("            %2u %S                  %12.5f Hz\n",
                            ui_number_of_channels,
                            ui_number_of_channels == 1 ? L"channel: " : L"channels:",
                            d_max_sampling_frequency);
                    }

                    printf("\n");

                    printf("        Number of ranges:                 %u\n",
                        s_adq_ai_module_info.m_uiNumberOfRanges);

                    for (unsigned int ui_index_range = 0; ui_index_range < s_adq_ai_module_info.m_uiNumberOfRanges; ui_index_range++)
                    {
                        wchar_t* wsz_physical_unit = L"????";

                        if (s_adq_ai_module_info.m_arrRanges[ui_index_range].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_VOLT)
                        {
                            wsz_physical_unit = L"Volt";
                        }
                        else if (s_adq_ai_module_info.m_arrRanges[ui_index_range].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_AMPERE)
                        {
                            wsz_physical_unit = L"Ampere";
                        }
                        else if(s_adq_ai_module_info.m_arrRanges[ui_index_range].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_OHM)
                        {
                            wsz_physical_unit = L"Ohm";
                        }

                        printf("            Range %d:\n"
                            "                  Digital Min.:           %u\n"
                            "                  Digital Max.:           %u\n"
                            "                  Unit:                   %S (0x%08X)\n"
                            "                  Physical Min.:          %7.6f\n"
                            "                  Physical Max.:          %7.6f\n",
                            ui_index_range,
                            s_adq_ai_module_info.m_arrRanges[ui_index_range].m_uiDigitalMin,
                            s_adq_ai_module_info.m_arrRanges[ui_index_range].m_uiDigitalMax,
                            wsz_physical_unit, s_adq_ai_module_info.m_arrRanges[ui_index_range].m_uiPhysicalUnit,
                            s_adq_ai_module_info.m_arrRanges[ui_index_range].m_dPhysicalMin,
                            s_adq_ai_module_info.m_arrRanges[ui_index_range].m_dPhysicalMax);
                    }

                    break;
                }
                case ADQ_MODULE_TYPE_AO:
                {
                    SADQAOModuleInfo s_adq_ao_module_info;

                    s_adq_ao_module_info.m_uiSize = sizeof(SADQAOModuleInfo);

                    s_adq_ao_module_info.m_uiID = SADQAOModuleInfo_ID;

                    ui_adq_error = adqGetModuleInfo(ui_index_board, ui_index_module, s_adq_general_module_info, sizeof(SADQAOModuleInfo), (unsigned char*)&s_adq_ao_module_info, ui_info_size);

                    if (ui_adq_error != ADQ_ERROR_NONE)
                    {
                        printf("        Board index: %3u Module index: %3u adqGetModuleInfo returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                        printf("Press any key to continue.\n");

                        _getch();

                        continue;
                    }

                    printf("        Number of ranges:                 %u\n",
                        s_adq_ao_module_info.m_uiNumberOfRanges);

                    for (unsigned int ui_index_range = 0; ui_index_range < s_adq_ao_module_info.m_uiNumberOfRanges; ui_index_range++)
                    {
                        wchar_t* wsz_physical_unit = L"????";

                        if (s_adq_ao_module_info.m_arrRanges[ui_index_range].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_VOLT)
                        {
                            wsz_physical_unit = L"Volt";
                        }
                        else if (s_adq_ao_module_info.m_arrRanges[ui_index_range].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_AMPERE)
                        {
                            wsz_physical_unit = L"Ampere";
                        }
                        else if(s_adq_ao_module_info.m_arrRanges[ui_index_range].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_OHM)
                        {
                            wsz_physical_unit = L"Ohm";
                        }

                        printf("            Range %d:\n"
                                "                  Digital Min.:           %u\n"
                                "                  Digital Max.:           %u\n"
                                "                  Unit:                   %S (0x%08X)\n"
                                "                  Physical Min.:          %7.6f\n"
                                "                  Physical Max.:          %7.6f\n",
                                ui_index_range,
                                s_adq_ao_module_info.m_arrRanges[ui_index_range].m_uiDigitalMin,
                                s_adq_ao_module_info.m_arrRanges[ui_index_range].m_uiDigitalMax,
                                wsz_physical_unit, s_adq_ao_module_info.m_arrRanges[ui_index_range].m_uiPhysicalUnit,
                                s_adq_ao_module_info.m_arrRanges[ui_index_range].m_dPhysicalMin,
                                s_adq_ao_module_info.m_arrRanges[ui_index_range].m_dPhysicalMax);
                    }

                    break;
                }
                case ADQ_MODULE_TYPE_I2C:
                {
                    SADQI2CModuleInfo s_adq_i2c_module_info;

                    s_adq_i2c_module_info.m_uiSize = sizeof(SADQI2CModuleInfo);

                    s_adq_i2c_module_info.m_uiID = SADQI2CModuleInfo_ID;

                    ui_adq_error = adqGetModuleInfo(ui_index_board, ui_index_module, s_adq_general_module_info, sizeof(SADQI2CModuleInfo), (unsigned char*)&s_adq_i2c_module_info, ui_info_size);

                    if (ui_adq_error != ADQ_ERROR_NONE)
                    {
                        printf("        Board index: %3u Module index: %3u adqGetModuleInfo returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                        printf("Press any key to continue.\n");

                        _getch();

                        continue;
                    }

                    printf("        Speed, Bits per Second:           %u\n"
                           "        Supports  7 bit addresses         %S\n"
                           "        Supports 10 bit addresses         %S\n",
                           s_adq_i2c_module_info.m_uiSpeedBitsPerSecond,
                           s_adq_i2c_module_info.m_bSupports7BitAddresses ? L"Yes" : L" No",
                           s_adq_i2c_module_info.m_bSupports10BitAddresses ? L"Yes" : L" No"    );

                    break;
                }
                case ADQ_MODULE_TYPE_PWM:
                {
                    SADQPWMModuleInfo s_adq_pwm_module_info;

                    s_adq_pwm_module_info.m_uiSize = sizeof(SADQPWMModuleInfo);

                    s_adq_pwm_module_info.m_uiID = SADQPWMModuleInfo_ID;

                    ui_adq_error = adqGetModuleInfo(ui_index_board, ui_index_module, s_adq_general_module_info, sizeof(SADQPWMModuleInfo), (unsigned char*)&s_adq_pwm_module_info, ui_info_size);

                    if (ui_adq_error != ADQ_ERROR_NONE)
                    {
                        printf("        Board index: %3u Module index: %3u adqGetModuleInfo returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                        printf("Press any key to continue.\n");

                        _getch();

                        continue;
                    }

                    printf("        Base frequency:                   %f Hz\n"
                           "        Minimum divider                   %u\n",
                           s_adq_pwm_module_info.m_dBaseFrequencyHz,
                           s_adq_pwm_module_info.m_uiMinimumDivider         );

                    double d_max_frequency = 0.0;

                    ui_adq_error = adqPWMGetMaximumFrequency(ui_index_board, ui_index_module, d_max_frequency);

                    if (ui_adq_error != ADQ_ERROR_NONE)
                    {
                        printf("        Board index: %3u Module index: %3u adqPWMGetMaximumFrequency returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                        printf("Press any key to continue.\n");

                        _getch();

                        continue;
                    }

                    printf("        Max frequency:                    %12.5f Hz\n",
                           d_max_frequency);

                    break;
                }
            }
        }
    }

    adqClose();

    printf("\n\n");

    printf("Press any key to terminate.\n");

    _getch();

    return 0;
}
