#include <windows.h>

#include <stdio.h>
#include <stdlib.h>

#include <tchar.h>

#include <conio.h>

#include "..\\adqDevice.h"

#ifdef WIN64

#pragma comment(lib, "..\\adqDevice_64.lib")

#else

#pragma comment(lib, "..\\adqDevice.lib")

#endif

unsigned int __stdcall adqAIReadContinuousCallback(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiNumberOfFramesAvailable, unsigned int* puiFrames, unsigned int uiError, void* pvUserContext, unsigned int uiReasonForCall);

struct SAIModule
{
    unsigned int    m_uiBoardIndex;

    unsigned int    m_uiModuleIndex;

    unsigned int    m_uiNumberOfChannels;

    bool            m_bSharedDataRate;

    bool            m_bRangeInfoPerChannel;

    unsigned int    m_uiNumberOfRanges;

    SADQAIRange     m_arrRanges[ADQ_MAX_AI_RANGES];
};

struct SAIChannelCalibration
{
    unsigned int    m_uiChannelIndex;

    double          m_dFactor;

    double          m_dConstant;

    wchar_t*        m_wszPhysicalUnit;
};

SAIChannelCalibration* parrsCalibrations;

unsigned int uiFrameLength;

int _tmain(int argc, _TCHAR* argv[])
{
   // Increase the size of the console buffer
    
    HANDLE  h_console = GetStdHandle(STD_OUTPUT_HANDLE);

    COORD coord;

    coord.X = 160;
    
    coord.Y = 9999;
    
    SetConsoleScreenBufferSize(h_console, coord);

    bool b_terminate = false;

    bool b_remote = false;

    wchar_t wsz_remote_name_or_IP_address[128];

    while(true)
    {
        printf( "1. Open local\n"
                "2. Open remote\n"
                "\n"
                "Your choice: 1 or 2 (RETURN to exit): ");

        char sz_response[8];

        gets_s(&sz_response[0], 8);

        printf("\n\n");

        if(sz_response[0] == 0)
        {
            b_terminate = true;
            
            break;
        }

        unsigned int ui_choice = 0;

        if( sscanf_s(&sz_response[0], "%u", &ui_choice) != 1)
        {
            continue;
        }

        if(ui_choice == 1)
        {
            break;
        }

        if(ui_choice == 2)
        {
            b_remote = true;

            printf("Name or IP-Address (RETURN to exit): ");

            _getws_s(&wsz_remote_name_or_IP_address[0], 128);

            printf("\n\n");

            if( wsz_remote_name_or_IP_address[0] == (wchar_t)0 )
            {
                b_terminate = true;
            }

            break;
        }
    }

    if(b_terminate)
    {
        printf("Press any key to exit.\n");

        _getch();

        exit(-1);
    }

    unsigned int ui_adq_error;
    
    if(b_remote)
    {
        wchar_t* pwc_user_name_or_address[2] = {&wsz_remote_name_or_IP_address[0],
                                                NULL                                };

        ui_adq_error = adqOpenEx(pwc_user_name_or_address);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("adqOpenEx returned 0x%08X\n\n", ui_adq_error);

            printf("Press any key to exit.\n");

            _getch();

            exit(-1);
        }
    }
    else
    {
        ui_adq_error = adqOpen();

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("adqOpen returned 0x%08X\n\n", ui_adq_error);

            printf("Press any key to exit.\n");

            _getch();

            exit(-1);
        }
    }

    unsigned int ui_number_of_boards;
    
    ui_adq_error = adqGetNumberOfBoards(ui_number_of_boards);

    if(ui_adq_error != ADQ_ERROR_NONE)
    {
        printf("adqGetNumberOfBoards returned 0x%08X\n\n", ui_adq_error);

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }

    if(ui_number_of_boards == 0)
    {
        printf("No ADQ boards found!\n\n");

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }

    printf("%3u ADQ Boards found\n", ui_number_of_boards);

    // Count the number of AI modules

    unsigned int ui_number_of_ai_modules = 0;

    for(unsigned int ui_index_board = 0; ui_index_board < ui_number_of_boards; ui_index_board++)
    {
        unsigned int ui_number_of_modules;

        ui_adq_error = adqGetNumberOfModules(ui_index_board, ui_number_of_modules);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("    Board index: %3u adqGetNumberOfModules returned 0x%08X\n\n", ui_index_board, ui_adq_error);

            printf("Press any key to continue.\n");

            _getch();

            continue;
        }

        for(unsigned int ui_index_module = 0; ui_index_module < ui_number_of_modules; ui_index_module++)
        {
            unsigned int ui_module_type;

            ui_adq_error = adqGetModuleType(ui_index_board, ui_index_module, ui_module_type);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("        Board index: %3u Module index: %3u adqGetModuleType returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            if(ui_module_type == ADQ_MODULE_TYPE_AI)
            {
                ++ui_number_of_ai_modules;
            }
        }
    }

    if(ui_number_of_ai_modules == 0)
    {
        printf("No AI modules found!\n\n");

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }

    unsigned int ui_max_number_of_ai_modules = ui_number_of_ai_modules;

    SAIModule* arr_ai_modules = new SAIModule[ui_max_number_of_ai_modules];

    ui_number_of_ai_modules = 0;

    for(unsigned int ui_index_board = 0; ui_index_board < ui_number_of_boards; ui_index_board++)
    {
        unsigned int ui_number_of_modules;

        ui_adq_error = adqGetNumberOfModules(ui_index_board, ui_number_of_modules);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("    Board index: %3u adqGetNumberOfModules returned 0x%08X\n\n", ui_index_board, ui_adq_error);

            printf("Press any key to continue.\n");

            _getch();

            continue;
        }

        for(unsigned int ui_index_module = 0; ui_index_module < ui_number_of_modules; ui_index_module++)
        {
            unsigned int ui_module_type;

            ui_adq_error = adqGetModuleType(ui_index_board, ui_index_module, ui_module_type);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("        Board index: %3u Module index: %3u adqGetModuleType returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            SADQGeneralModuleInfo s_adq_general_module_info;

            s_adq_general_module_info.m_uiSize = sizeof(SADQGeneralModuleInfo);

            s_adq_general_module_info.m_uiID = SADQGeneralModuleInfo_ID;

            if(ui_module_type == ADQ_MODULE_TYPE_AI)
            {
                SADQAIModuleInfo s_adq_ai_module_info;

                s_adq_ai_module_info.m_uiSize = sizeof(SADQAIModuleInfo);

                s_adq_ai_module_info.m_uiID = SADQAIModuleInfo_ID;

                unsigned int ui_info_size;

                ui_adq_error = adqGetModuleInfo(ui_index_board, ui_index_module, s_adq_general_module_info, sizeof(SADQAIModuleInfo), (unsigned char*)&s_adq_ai_module_info, ui_info_size);

                if(ui_adq_error != ADQ_ERROR_NONE)
                {
                    printf("        Board index: %3u Module index: %3u adqGetModuleInfo returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                    printf("Press any key to continue.\n");

                    _getch();

                    continue;
                }

                arr_ai_modules[ui_number_of_ai_modules].m_uiBoardIndex = ui_index_board;

                arr_ai_modules[ui_number_of_ai_modules].m_uiModuleIndex = ui_index_module;

                arr_ai_modules[ui_number_of_ai_modules].m_uiNumberOfChannels = s_adq_ai_module_info.m_uiNumberOfChannels;

                arr_ai_modules[ui_number_of_ai_modules].m_bSharedDataRate = s_adq_ai_module_info.m_bSharedDataRate;

                arr_ai_modules[ui_number_of_ai_modules].m_bRangeInfoPerChannel = s_adq_ai_module_info.m_bRangeInfoPerChannel;

                arr_ai_modules[ui_number_of_ai_modules].m_uiNumberOfRanges = s_adq_ai_module_info.m_uiNumberOfRanges;

                memcpy( &arr_ai_modules[ui_number_of_ai_modules].m_arrRanges[0],  &s_adq_ai_module_info.m_arrRanges[0], arr_ai_modules[ui_number_of_ai_modules].m_uiNumberOfRanges * sizeof(SADQAIRange) );

                ++ui_number_of_ai_modules;
            }
        }
    }

    while(true)
    {
        printf("Please choose an AI module:\n\n");

        unsigned int ui_index_ai_module;

        for(ui_index_ai_module = 0; ui_index_ai_module < ui_number_of_ai_modules; ui_index_ai_module++)
        {
            printf("%2u.  Board index: %d  Module index: %-2d  Number of channels: %-2d  Number of ranges: %-2d\n",
                   ui_index_ai_module + 1,
                   arr_ai_modules[ui_index_ai_module].m_uiBoardIndex,
                   arr_ai_modules[ui_index_ai_module].m_uiModuleIndex,
                   arr_ai_modules[ui_index_ai_module].m_uiNumberOfChannels,
                   arr_ai_modules[ui_index_ai_module].m_uiNumberOfRanges                                                            );
        }

        printf("\n");

        printf("Your choice 1 - %d (RETURN to exit): ",
               ui_number_of_ai_modules                  );

        char sz_response[256];

        gets_s(&sz_response[0], 8);

        printf("\n\n");

        if(sz_response[0] == 0)
        {
            break;
        }

        if( sscanf_s(&sz_response[0], "%u", &ui_index_ai_module) != 1)
        {
            continue;
        }

        if( (ui_index_ai_module < 1)||(ui_index_ai_module > ui_number_of_ai_modules) )
        {
            continue;
        }

        --ui_index_ai_module;

        unsigned int ui_board_index = arr_ai_modules[ui_index_ai_module].m_uiBoardIndex;

        unsigned int ui_module_index = arr_ai_modules[ui_index_ai_module].m_uiModuleIndex;

        SADQAIChannel* p_arr_s_channels = new SADQAIChannel[ arr_ai_modules[ui_index_ai_module].m_uiNumberOfChannels ];

        memset( &p_arr_s_channels[0], 0, arr_ai_modules[ui_index_ai_module].m_uiNumberOfChannels * sizeof(SADQAIChannel) );

        bool b_continue = false;

        while(true)
        {
            if(arr_ai_modules[ui_index_ai_module].m_uiNumberOfChannels < 4)
            {
                printf( "The module has %u channels, please enter a comma separated channel list - e.g 0, %u (RETURN to exit): ",
                        arr_ai_modules[ui_index_ai_module].m_uiNumberOfChannels,
                        arr_ai_modules[ui_index_ai_module].m_uiNumberOfChannels - 1                                                 );
            }
            else
            {
                printf( "The module has %u channels, please enter a comma separated channel list - e.g 0,2, %u (RETURN to exit): ",
                        arr_ai_modules[ui_index_ai_module].m_uiNumberOfChannels,
                        arr_ai_modules[ui_index_ai_module].m_uiNumberOfChannels - 1                                                 );
            }
                 
            gets_s(&sz_response[0], 256);

            printf("\n\n");

            if(sz_response[0] == 0)
            {
                b_continue = true;
                    
                break;
            }

            char* p_c_ch_context;
            
            char* p_ch_channel = strtok_s(&sz_response[0], ",.", &p_c_ch_context);
            
            while(p_ch_channel != NULL)
            {
                unsigned int ui_index_channel;

                if( sscanf_s(p_ch_channel, "%u", &ui_index_channel) != 1)
                {
                    continue;
                }

                if(ui_index_channel >= arr_ai_modules[ui_index_ai_module].m_uiNumberOfChannels)
                {
                    printf("Invalid channel: %u - Press any key to continue\n\n", ui_index_channel);

                    b_continue = true;

                    break;
                }

                p_arr_s_channels[ui_index_channel].m_bAcquireChannel = true;

                p_arr_s_channels[ui_index_channel].m_uiRangeIndex = 0;

                p_ch_channel = strtok_s(NULL, ",.", &p_c_ch_context);
            }

            break;
        }

        if(b_continue)
        {
            delete p_arr_s_channels;

            continue;
        }

        if(arr_ai_modules[ui_index_ai_module].m_bRangeInfoPerChannel)
        {
            printf("        Board index: %3u Module index: %3u - Not all the channels have the same ranges - Program must be extended\n\n", arr_ai_modules[ui_index_ai_module].m_uiBoardIndex, arr_ai_modules[ui_index_ai_module].m_uiBoardIndex, ui_adq_error);

            printf("Press any key to continue.\n");

            _getch();

            delete p_arr_s_channels;

            continue;
        }

        unsigned int ui_range_index;
        ui_range_index = 0;

        if(arr_ai_modules[ui_index_ai_module].m_uiNumberOfRanges > 1)
        {
            b_continue = false;
            
            while(true)
            {
                printf("The channel has the following input ranges:\n\n");
                 
                for(unsigned int ui_index_range = 0; ui_index_range < arr_ai_modules[ui_index_ai_module].m_uiNumberOfRanges; ui_index_range++)
                {
                    wchar_t* wsz_physical_unit;

                    if(arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_VOLT)
                    {
                        wsz_physical_unit = L"Volt";
                    }
                    else if(arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_AMPERE)
                    {
                        wsz_physical_unit = L"Ampere";
                    }
                    else // if(arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_OHM)
                    {
                        wsz_physical_unit = L"Ohm";
                    }

                    printf( "%2d.  Digital Min.: %u  Digital Max.: %u  Unit: %6S  Physical Min.: %7.4f  Physical Max.: %7.4f\n",
                            ui_index_range,
                            arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_uiDigitalMin,
                            arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_uiDigitalMax,
                            wsz_physical_unit,
                            arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_dPhysicalMin,
                            arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_dPhysicalMax                                     );
                }

                printf("\n");

                printf("Your choice 0 - %u (RETURN to exit): ",
                        arr_ai_modules[ui_index_ai_module].m_uiNumberOfRanges - 1  );

                char sz_response[8];

                gets_s(&sz_response[0], 8);

                printf("\n\n");

                if(sz_response[0] == 0)
                {
                    b_continue = true;
                    
                    break;
                }

                if( sscanf_s(&sz_response[0], "%u", &ui_range_index) != 1)
                {
                    continue;
                }

                if(ui_range_index >= arr_ai_modules[ui_index_ai_module].m_uiNumberOfRanges)
                {
                    continue;
                }

                printf("All channels will be measured using range %u\n\n"
                       "With the API it is possible to use a different range for each channel\n\n",
                       ui_range_index                                                               );

                break;
            }

            if(b_continue)
            {
                delete p_arr_s_channels;

                continue;
            }
        }

        if(ui_range_index != 0)
        {
            for(unsigned int ui_index_channel = 0; ui_index_channel < arr_ai_modules[ui_index_ai_module].m_uiNumberOfChannels; ui_index_channel++)
            {
                p_arr_s_channels[ui_index_channel].m_uiRangeIndex = ui_range_index;
            }
        }

        int i_required_frequency;

        char buffer[64];

        b_continue = false;

        while(true)
        {
            printf("Required frequency (sampling frequency) in Hz (RETURN to terminate): ");

            gets_s(&buffer[0], 64);

            printf("\n\n");
        
            if(buffer[0] == 0)
            {
                b_continue = true;
            
                break;
            }
        
            if(sscanf_s(&buffer[0], "%d", &i_required_frequency) != 1 )
            {
                continue;
            }

            if(i_required_frequency < 1)
            {
                continue;
            }

            break;
        }

        if(b_continue)
        {
            delete parrsCalibrations;

            delete p_arr_s_channels;

            continue;
        }

        double d_nominal_frequency = (double)i_required_frequency;

        double d_actual_frequency;

        ui_adq_error = adqAISetChannelListAndFrequency(arr_ai_modules[ui_index_ai_module].m_uiBoardIndex, arr_ai_modules[ui_index_ai_module].m_uiModuleIndex, p_arr_s_channels, arr_ai_modules[ui_index_ai_module].m_uiNumberOfChannels, d_nominal_frequency, d_actual_frequency);
        
        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf( "        Board index: %3u Module index: %3u - adqAISetChannelListAndFrequency returned 0x%08X\n\n", 
                    arr_ai_modules[ui_index_ai_module].m_uiBoardIndex,
                    arr_ai_modules[ui_index_ai_module].m_uiModuleIndex,
                    ui_adq_error                                                                                        );

           delete p_arr_s_channels;

           printf("Press any key to continue.\n");

           _getch();

            continue;
        }

        printf("Nominal frequency %f Hz    Actual frequency: %f\n\n",  d_nominal_frequency, d_actual_frequency);

        uiFrameLength = 0;

        for(unsigned int ui_index_channel = 0; ui_index_channel < arr_ai_modules[ui_index_ai_module].m_uiNumberOfChannels; ui_index_channel++)
        {
            if(p_arr_s_channels[ui_index_channel].m_bAcquireChannel)
            {
                ++uiFrameLength;
            }
        }

        parrsCalibrations = new SAIChannelCalibration[uiFrameLength];

        unsigned int ui_frame_index = 0;

        for(unsigned int ui_index_channel = 0; ui_index_channel < arr_ai_modules[ui_index_ai_module].m_uiNumberOfChannels; ui_index_channel++)
        {
            if(p_arr_s_channels[ui_index_channel].m_bAcquireChannel)
            {
                parrsCalibrations[ui_frame_index].m_uiChannelIndex = ui_index_channel;

                unsigned int ui_index_range = p_arr_s_channels[ui_index_channel].m_uiRangeIndex;

                parrsCalibrations[ui_frame_index].m_dFactor = (arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_dPhysicalMax - arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_dPhysicalMin) / 
                                            (double)(arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_uiDigitalMax - arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_uiDigitalMin);
        
                parrsCalibrations[ui_frame_index].m_dConstant = arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_dPhysicalMin - (double)arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_uiDigitalMin * parrsCalibrations[ui_frame_index].m_dFactor; 

                if(arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_VOLT)
                {
                    parrsCalibrations[ui_frame_index].m_wszPhysicalUnit = L"Volt";
                }
                else if(arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_AMPERE)
                {
                    parrsCalibrations[ui_frame_index].m_wszPhysicalUnit = L"Ampere";
                }
                else // if(arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_OHM)
                {
                    parrsCalibrations[ui_frame_index].m_wszPhysicalUnit = L"Ohm";
                }

                ++ui_frame_index;
            }
        }

        printf("Parameters for adqAIReadContinuousStart:\n\n");

        b_continue = false;

        unsigned int ui_notify_after_frames = 100;

        while(true)
        {
            printf("Notify after frames, 1 - 50000 (RETURN to terminate): ");

            gets_s(&buffer[0], 64);

            printf("\n\n");
        
            if(buffer[0] == 0)
            {
                b_continue = true;
            
                break;
            }
        
            if( (sscanf_s(&buffer[0], "%u", &ui_notify_after_frames) != 1 ) )
            {
                continue;
            }

            if( (ui_notify_after_frames < 1)||(ui_notify_after_frames > 50000) )
            {
                continue;
            }

            break;
        }

        if(b_continue)
        {
            delete parrsCalibrations;

            delete p_arr_s_channels;

            continue;
        }

        printf( "Press any key to call adqAIReadContinuousStart and start an acquisition from board index: %u, module index: %u at %f Hz\n"
                "The software will be notified each time %u frames have been acquired\n\n"
                "Each frame consists of values from the following channels: ",
                ui_board_index, 
                ui_module_index,
                d_actual_frequency,
                ui_notify_after_frames                                                                                                    );

        for(unsigned int ui_frame_index = 0; ui_frame_index < uiFrameLength; ui_frame_index++)
        {
            printf("%d ", parrsCalibrations[ui_frame_index].m_uiChannelIndex);
        }

        printf("\n\n");

        printf( "After this the callback function adqAIReadContinuousCallback will be called automatically when data is available\n\n"
                "Press ESC at any time to end the acquisition\n\n"                                                                  );

        _getch();

        ui_adq_error = adqAIReadContinuousStart( ui_board_index, ui_module_index, ui_notify_after_frames, 0, adqAIReadContinuousCallback, &arr_ai_modules[ui_index_ai_module] );

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("adqAIReadContinuousStart returned 0x%08X\n\n", ui_adq_error);

            printf("Press any key to continue.\n");

            _getch();

            delete parrsCalibrations;

            delete p_arr_s_channels;

            continue;
        }

        while(true)
        {
            if( _kbhit() )
            {
                char ch_key = _getch();

                if(ch_key == 27)
                {
                    break;
                }
            }
        }

        ui_adq_error = adqStop(ui_board_index, ui_module_index);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("adqStop returned 0x%08X\n\n", ui_adq_error);

            printf("Press any key to continue.\n");

            _getch();

            continue;
        }

        printf("Acquisition terminated\n\n");

        printf("Press any key to continue\n\n");

        _getch();

        delete parrsCalibrations;

        delete p_arr_s_channels;

        continue;
    }

    printf("\n\n");

    delete []arr_ai_modules;

    adqClose();

    printf("Press any key to terminate.\n");

    _getch();

    return 0;
}

unsigned int __stdcall adqAIReadContinuousCallback(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiNumberOfFramesAvailable, unsigned int* puiFrames, unsigned int uiError, void* pvUserContext, unsigned int uiReasonForCall)
{
    SAIModule* p_s_adq_ai_module = (SAIModule*)pvUserContext;

    printf("adqAIReadContinuousCallback called - uiNumberOfFramesAvailable: %u uiError: %u, uiReasonForCall: %u\n\n", uiNumberOfFramesAvailable, uiError, uiReasonForCall);

    if( (uiReasonForCall & ADQ_CALLED_FOR_START) == ADQ_CALLED_FOR_START )
    {
        printf("Called for start\n\n");
    }

    if( (uiReasonForCall & ADQ_CALLED_FOR_DATA) == ADQ_CALLED_FOR_DATA )
    {
        printf("adqAIReadContinuousCallback returned %u frames\n", uiNumberOfFramesAvailable);

        printf("\n                ");

        for(unsigned int ui_index_channel = 0; ui_index_channel < uiFrameLength; ui_index_channel++)
        {
            printf("Channel %3d         ", parrsCalibrations[ui_index_channel].m_uiChannelIndex);
        }

        printf("\n                ");

        for(unsigned int ui_index_channel = 0; ui_index_channel < uiFrameLength; ui_index_channel++)
        {
            printf("-----------         ");
        }

        printf("\n\n                ");

        for(unsigned int ui_index_channel = 0; ui_index_channel < uiFrameLength; ui_index_channel++)
        {
            printf("Digital   %7S   ", parrsCalibrations[ui_index_channel].m_wszPhysicalUnit);
        }

        printf("\n                ");

        for(unsigned int ui_index_channel = 0; ui_index_channel < uiFrameLength; ui_index_channel++)
        {
            printf("-------   -------   ");
        }

        printf("\n");

        if(uiNumberOfFramesAvailable > 0)
        {
            printf("First frame:    ");

            for(unsigned int ui_index_channel = 0; ui_index_channel < uiFrameLength; ui_index_channel++)
            {
                unsigned int ui_data = puiFrames[ui_index_channel];

                double d_data = (double)ui_data * parrsCalibrations[ui_index_channel].m_dFactor + parrsCalibrations[ui_index_channel].m_dConstant;

                printf("%7u   %7.3f   ",
                        ui_data,
                        d_data              );
            }

            printf("\n");
        }

        if(uiNumberOfFramesAvailable > 1)
        {
            printf("Last frame:     ");

            for(unsigned int ui_index_channel = 0; ui_index_channel < uiFrameLength; ui_index_channel++)
            {
                unsigned int ui_data = puiFrames[ (uiNumberOfFramesAvailable - 1) * uiFrameLength + ui_index_channel];

                double d_data = (double)ui_data * parrsCalibrations[ui_index_channel].m_dFactor + parrsCalibrations[ui_index_channel].m_dConstant;

                printf("%7u   %7.3f   ",
                        ui_data,
                        d_data              );
            }

            printf("\n");
        }

        printf("\n\n\n");
    }

    if( (uiReasonForCall & ADQ_CALLED_FOR_STOP) == ADQ_CALLED_FOR_STOP )
    {
        printf("Called for stop\n\n");
    }

    return 0;
}



