#include <windows.h>

#include <stdio.h>
#include <stdlib.h>

#include <tchar.h>

#include <conio.h>

#include <memory.h>

#include <windows.h>

#include "..\\adqDevice.h"

#ifdef WIN64

#pragma comment(lib, "..\\adqDevice_64.lib")

#else

#pragma comment(lib, "..\\adqDevice.lib")

#endif

unsigned int __stdcall adqIRQCallback(unsigned int uiBoardIndex, unsigned int uiModuleIndex, SADQIRQStatus* psIRQStatus, unsigned int uiError, void* pvUserContext);

struct SIRQSource
{
    unsigned int    m_uiBoardIndex;

    unsigned int    m_uiModuleIndex;

    unsigned int    m_uiModuleType;

    SADQIRQInfo     m_sADQIRQInfo;
};

int _tmain(int argc, _TCHAR* argv[])
{
   // Increase the size of the console buffer
    
    HANDLE  h_console = GetStdHandle(STD_OUTPUT_HANDLE);

    COORD coord;

    coord.X = 160;
    
    coord.Y = 9999;
    
    SetConsoleScreenBufferSize(h_console, coord);

    bool b_terminate = false;

    bool b_remote = false;

    wchar_t wsz_remote_name_or_IP_address[128];

    while(true)
    {
        printf( "1. Open local\n"
                "2. Open remote\n"
                "\n"
                "Your choice: 1 or 2 (RETURN to exit): ");

        char sz_response[8];

        gets_s(&sz_response[0], 8);

        printf("\n\n");

        if(sz_response[0] == 0)
        {
            b_terminate = true;
            
            break;
        }

        unsigned int ui_choice = 0;

        if( sscanf_s(&sz_response[0], "%u", &ui_choice) != 1)
        {
            continue;
        }

        if(ui_choice == 1)
        {
            break;
        }

        if(ui_choice == 2)
        {
            b_remote = true;

            printf("Name or IP-Address (RETURN to exit): ");

            _getws_s(&wsz_remote_name_or_IP_address[0], 128);

            printf("\n\n");

            if( wsz_remote_name_or_IP_address[0] == (wchar_t)0 )
            {
                b_terminate = true;
            }

            break;
        }
    }

    if(b_terminate)
    {
        printf("Press any key to exit.\n");

        _getch();

        exit(-1);
    }

    unsigned int ui_adq_error;
    
    if(b_remote)
    {
        wchar_t* pwc_user_name_or_address[2] = {&wsz_remote_name_or_IP_address[0],
                                                NULL                                };

        ui_adq_error = adqOpenEx(pwc_user_name_or_address);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("adqOpenEx returned ERROR 0x%08X\n\n", ui_adq_error);

            printf("Press any key to exit.\n");

            _getch();

            exit(-1);
        }
    }
    else
    {
        ui_adq_error = adqOpen();

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("adqOpen returned ERROR 0x%08X\n\n", ui_adq_error);

            printf("Press any key to exit.\n");

            _getch();

            exit(-1);
        }
    }

    unsigned int ui_number_of_boards;
    
    ui_adq_error = adqGetNumberOfBoards(ui_number_of_boards);

    if(ui_adq_error != ADQ_ERROR_NONE)
    {
        printf("adqGetNumberOfBoards returned ERROR 0x%08X\n\n", ui_adq_error);

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }

    if(ui_number_of_boards == 0)
    {
        printf("No ALLDAQ boards found!\n\n");

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }

    printf("%3u ADQ Boards found\n", ui_number_of_boards);

    // Count the number of modules which can be used as an IRQ source

    unsigned int ui_number_of_irq_sources = 0;

    for(unsigned int ui_index_board = 0; ui_index_board < ui_number_of_boards; ui_index_board++)
    {
        unsigned int ui_number_of_modules;

        ui_adq_error = adqGetNumberOfModules(ui_index_board, ui_number_of_modules);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("    Board index: %3u adqGetNumberOfModules returned ERROR 0x%08X\n\n", ui_index_board, ui_adq_error);

            printf("Press any key to continue.\n");

            _getch();

            continue;
        }

        for(unsigned int ui_index_module = 0; ui_index_module < ui_number_of_modules; ui_index_module++)
        {
            SADQGeneralModuleInfo s_module_info;

            s_module_info.m_uiSize = sizeof(SADQGeneralModuleInfo);

            s_module_info.m_uiID = SADQGeneralModuleInfo_ID;

            unsigned int ui_info_size;

            ui_adq_error = adqGetModuleInfo(ui_index_board, ui_index_module, s_module_info, 0, NULL, ui_info_size);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("        Board index: %3u Module index: %3u adqGetModuleType returned ERROR 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            if(s_module_info.m_bIsIRQSource)
            {
                ++ui_number_of_irq_sources;
            }
        }
    }

    if(ui_number_of_irq_sources == 0)
    {
        printf("No modules found which can serve as an IRQ source.\n\n");

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }

    unsigned int ui_max_number_of_irq_sources = ui_number_of_irq_sources;

    SIRQSource* arr_irq_sources = new SIRQSource[ui_max_number_of_irq_sources];

    ui_number_of_irq_sources = 0;

    for(unsigned int ui_index_board = 0; ui_index_board < ui_number_of_boards; ui_index_board++)
    {
        unsigned int ui_number_of_modules;

        ui_adq_error = adqGetNumberOfModules(ui_index_board, ui_number_of_modules);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("    Board index: %3u adqGetNumberOfModules returned ERROR 0x%08X\n\n", ui_index_board, ui_adq_error);

            printf("Press any key to continue.\n");

            _getch();

            continue;
        }

        for(unsigned int ui_index_module = 0; ui_index_module < ui_number_of_modules; ui_index_module++)
        {
            SADQGeneralModuleInfo s_module_info;

            s_module_info.m_uiSize = sizeof(SADQGeneralModuleInfo);

            s_module_info.m_uiID = SADQGeneralModuleInfo_ID;

            unsigned int ui_info_size;

            ui_adq_error = adqGetModuleInfo(ui_index_board, ui_index_module, s_module_info, 0, NULL, ui_info_size);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("        Board index: %3u Module index: %3u adqGetModuleType returned ERROR 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            if(s_module_info.m_bIsIRQSource)
            {
                if(ui_number_of_irq_sources < ui_max_number_of_irq_sources)
                {
                    arr_irq_sources[ui_number_of_irq_sources].m_sADQIRQInfo.m_uiSize = sizeof(SADQIRQInfo);
                    
                    arr_irq_sources[ui_number_of_irq_sources].m_sADQIRQInfo.m_uiID = SADQIRQInfo_ID;
                    
                    ui_adq_error = adqGetIRQInfo( ui_index_board, ui_index_module, arr_irq_sources[ui_number_of_irq_sources].m_sADQIRQInfo );

                    if(ui_adq_error != ADQ_ERROR_NONE)
                    {
                        printf("        Board index: %3u Module index: %3u adqGetIRQInfo returned ERROR 0x%08X\n\n", ui_index_board + 1, ui_index_module + 1, ui_adq_error);

                        printf("Press any key to continue.\n");

                        _getch();

                        continue;
                    }

                    arr_irq_sources[ui_number_of_irq_sources].m_uiBoardIndex = ui_index_board;

                    arr_irq_sources[ui_number_of_irq_sources].m_uiModuleIndex = ui_index_module;

                    arr_irq_sources[ui_number_of_irq_sources].m_uiModuleType = s_module_info.m_uiModuleType;

                    ++ui_number_of_irq_sources;
                }
            }
        }
    }

    while(true)
    {
        printf("The following modules can serve as an IRQ source - Please choose one:\n\n");

        unsigned int ui_index_irq_source;

        for(ui_index_irq_source = 0; ui_index_irq_source < ui_number_of_irq_sources; ui_index_irq_source++)
        {
            char* sz_module_type;

            if(arr_irq_sources[ui_index_irq_source].m_uiModuleType == ADQ_MODULE_TYPE_DI)
            {
                sz_module_type = " DI";
            }
            else if(arr_irq_sources[ui_index_irq_source].m_uiModuleType == ADQ_MODULE_TYPE_DO)
            {
                sz_module_type = " DO";
            }
            else if(arr_irq_sources[ui_index_irq_source].m_uiModuleType == ADQ_MODULE_TYPE_DIO)
            {
                sz_module_type = "DIO";
            }
            else if (arr_irq_sources[ui_index_irq_source].m_uiModuleType == ADQ_MODULE_TYPE_EXTINT)
            {
                sz_module_type = "EXTINT";
            }
            else if (arr_irq_sources[ui_index_irq_source].m_uiModuleType == ADQ_MODULE_TYPE_AI)
            {
                sz_module_type = "AI";
            }
            else if (arr_irq_sources[ui_index_irq_source].m_uiModuleType == ADQ_MODULE_TYPE_AO)
            {
                sz_module_type = "AO";
            }
            else if (arr_irq_sources[ui_index_irq_source].m_uiModuleType == ADQ_MODULE_TYPE_CNTR)
            {
                sz_module_type = "CNTR";
            }
            else
            {
                sz_module_type = "Unknown";
            }

            printf("%2u.  Board index: %d  Module index: %-2d  Module type: %s\n"
                    "     IRQ Description: %S\n\n",
                   ui_index_irq_source + 1,
                   arr_irq_sources[ui_index_irq_source].m_uiBoardIndex,
                   arr_irq_sources[ui_index_irq_source].m_uiModuleIndex,
                   sz_module_type,
                   &arr_irq_sources[ui_index_irq_source].m_sADQIRQInfo.m_wszIRQDescription[0]);
        }

        printf("\n\n");

        printf("Your choice 1 - %d (RETURN to exit): ",
               ui_number_of_irq_sources                 );

        char sz_response[8];

        gets_s(&sz_response[0], 8);

        printf("\n\n");

        if(sz_response[0] == 0)
        {
            break;
        }

        if( sscanf_s(&sz_response[0], "%u", &ui_index_irq_source) != 1)
        {
            continue;
        }

        if( (ui_index_irq_source < 1) || (ui_index_irq_source > ui_number_of_irq_sources) )
        {
            continue;
        }

        --ui_index_irq_source;

        bool b_bit_change_irq = false;

        bool b_pattern_irq = false;

        bool b_error_or_recovery_irq = false;

        bool b_external_irq = false;

        unsigned int ui_number_of_irq_types = 0;

        if(arr_irq_sources[ui_index_irq_source].m_sADQIRQInfo.m_bSupportsBitChange)
        {
            b_bit_change_irq = true;

            ++ui_number_of_irq_types;
        }

        if(arr_irq_sources[ui_index_irq_source].m_sADQIRQInfo.m_bSupportsBitPatternCompare)
        {
            b_pattern_irq = true;

            ++ui_number_of_irq_types;
        }

        if( (arr_irq_sources[ui_index_irq_source].m_sADQIRQInfo.m_uiSupportsPortStatusError != 0x00000000) 
            || (arr_irq_sources[ui_index_irq_source].m_sADQIRQInfo.m_uiSupportsFaultRecovery != 0x00000000) )
        {
            b_error_or_recovery_irq = true;

            ++ui_number_of_irq_types;
        }

        if(arr_irq_sources[ui_index_irq_source].m_sADQIRQInfo.m_uiSupportsExternal != 0x00000000)
        {
            b_external_irq = true;

            ++ui_number_of_irq_types;
        }

        if(ui_number_of_irq_types == 0)
        {
            printf("Test program must be updated for this module - Press any key to continue\n");

            _getch();

            continue;
        }

        if(ui_number_of_irq_types > 1)
        {
            // User must choose the type of IRQ

            bool b_terminate = false;

            while(true)
            {
                printf("This module supports several types of interrupt - Please choose one:\n\n");

                unsigned int ui_index_irq_type = 0;

                if(b_bit_change_irq)
                {
                    printf("%2u.  Bit change\n",
                           ui_index_irq_type + 1  );

                    ++ui_index_irq_type;
                }

                if(b_pattern_irq)
                {
                    printf("%2u.  Bit pattern compare\n",
                           ui_index_irq_type + 1  );

                    ++ui_index_irq_type;
                }

                if(b_error_or_recovery_irq)
                {
                    printf("%2u.  Port status\n",
                           ui_index_irq_type + 1          );

                    ++ui_index_irq_type;
                }

                if(b_external_irq)
                {
                    printf("%2u.  External IRQ\n",
                           ui_index_irq_type + 1          );

                    ++ui_index_irq_type;
                }

                printf("\n\n");

                printf("Your choice 1 - %d (RETURN to exit): ",
                       ui_number_of_irq_types                   );

                char sz_response[8];

                gets_s(&sz_response[0], 8);

                printf("\n\n");

                if(sz_response[0] == 0)
                {
                    b_terminate = true;

                    break;
                }

                if( sscanf_s(&sz_response[0], "%u", &ui_index_irq_type) != 1)
                {
                    continue;
                }

                if( (ui_index_irq_type < 1)||(ui_index_irq_type > ui_number_of_irq_types) )
                {
                    continue;
                }

                --ui_index_irq_type;

                // Set those types of IRQ to 'false' which were NOT chosen.

                unsigned int ui_number_irq_type = 0;

                if(b_bit_change_irq)
                {
                    if(ui_number_irq_type != ui_index_irq_type)
                    {
                        b_bit_change_irq = false;
                    }

                    ++ui_number_irq_type;
                }

                if(b_pattern_irq)
                {
                    if(ui_number_irq_type != ui_index_irq_type)
                    {
                        b_pattern_irq = false;
                    }

                    ++ui_number_irq_type;
                }

                if(b_error_or_recovery_irq)
                {
                    if(ui_number_irq_type != ui_index_irq_type)
                    {
                        b_error_or_recovery_irq = false;
                    }

                    ++ui_number_irq_type;
                }

                if(b_external_irq)
                {
                    if(ui_number_irq_type != ui_index_irq_type)
                    {
                        b_external_irq = false;
                    }

                    ++ui_number_irq_type;
                }

                break;
            }

            if(b_terminate)
            {
                continue;
            }
        }

        // User must enter the parameters for the IRQ

        SADQIRQEnable s_adq_irq_enable;

        memset( &s_adq_irq_enable, 0, sizeof(SADQIRQEnable) );

        s_adq_irq_enable.m_uiSize = sizeof(SADQIRQEnable);

        s_adq_irq_enable.m_uiID = SADQIRQEnable_ID;

        s_adq_irq_enable.m_bResetIRQCount = true;

        if(b_bit_change_irq)
        {
            unsigned int ui_bit_change_rising_edge = 0x00000000;

            unsigned int ui_bit_change_falling_edge = 0x00000000;

            bool b_terminate = false;
                
            while(true)
            {
                printf( "Mask - Rising Edge (RETURN to exit): 0x");

                char sz_response[32];

                gets_s(&sz_response[0], 32);

                printf("\n\n");

                if(sz_response[0] == 0)
                {
                    b_terminate = true;
                        
                    break;
                }

                if( sscanf_s(&sz_response[0], "%X", &ui_bit_change_rising_edge) != 1)
                {
                    continue;
                }

                break;
            }

            if(b_terminate)
            {
                continue;
            }

            while(true)
            {
                printf( "Mask - Falling Edge (RETURN to exit): 0x");

                char sz_response[32];

                gets_s(&sz_response[0], 32);

                printf("\n\n");

                if(sz_response[0] == 0)
                {
                    b_terminate = true;
                        
                    break;
                }

                if( sscanf_s(&sz_response[0], "%X", &ui_bit_change_falling_edge) != 1)
                {
                    continue;
                }

                break;
            }

            if(b_terminate)
            {
                continue;
            }

            s_adq_irq_enable.m_uiPortBitChangeRisingEdge = ui_bit_change_rising_edge;

            s_adq_irq_enable.m_uiPortBitChangeFallingEdge = ui_bit_change_falling_edge;
        }
        else if(b_pattern_irq)
        {
            unsigned int ui_bit_pattern = 0x00000000;
            
            bool b_terminate = false;
            
            while(true)
            {
                printf( "Bit pattern compare (RETURN to exit): 0x");

                char sz_response[32];

                gets_s(&sz_response[0], 32);

                printf("\n\n");

                if(sz_response[0] == 0)
                {
                    b_terminate = true;
                        
                    break;
                }

                if( sscanf_s(&sz_response[0], "%X", &ui_bit_pattern) != 1)
                {
                    continue;
                }

                break;
            }

            if(b_terminate)
            {
                continue;
            }

            s_adq_irq_enable.m_bPortBitPatternCompare = true;

            s_adq_irq_enable.m_uiPortBitPatternValue = ui_bit_pattern;
        }
        else if(b_error_or_recovery_irq)
        {
            bool b_terminate = false;
            
            while(true)
            {
                printf( "Port status for bit 0\n"
                        "1. Error\n"
                        "2. Fault recovery\n"
                        "3. Any Error or Recovery\n\n\n"
                        "Your choice 1 - 3 (RETURN to exit): "  );

                char sz_response[8];

                gets_s(&sz_response[0], 8);

                printf("\n\n");

                if(sz_response[0] == 0)
                {
                    b_terminate = true;
                        
                    break;
                }

                unsigned int ui_number_choice;

                if( sscanf_s(&sz_response[0], "%u", &ui_number_choice) != 1)
                {
                    continue;
                }

                if( (ui_number_choice < 1)||(ui_number_choice > 3) )
                {
                    continue;
                }

                if(ui_number_choice == 1)
                {
                    s_adq_irq_enable.m_uiPortStatusError = 0x00000001;
                }
                else if(ui_number_choice == 2)
                {
                    s_adq_irq_enable.m_uiFaultRecovery = 0x00000001;
                }
                else // if(ui_index_irq_source == 3)
                {
                    s_adq_irq_enable.m_uiPortStatusError = 0x00000001;

                    s_adq_irq_enable.m_uiFaultRecovery = 0x00000001;
                }

                break;
            }

            if(b_terminate)
            {
                continue;
            }
        }
        else if(b_external_irq)
        {
            unsigned int ui_external_rising_edge = 0x00000000;

            unsigned int ui_external_falling_edge = 0x00000000;

            bool b_terminate = false;
                
            while(true)
            {
                printf("Enabled interrupts - Rising Edge (RETURN to exit): 0x");

                char sz_response[32];

                gets_s(&sz_response[0], 32);

                printf("\n\n");

                if(sz_response[0] == 0)
                {
                    b_terminate = true;
                        
                    break;
                }

                if( sscanf_s(&sz_response[0], "%X", &ui_external_rising_edge) != 1)
                {
                    continue;
                }

                if( (ui_external_rising_edge & ~arr_irq_sources[ui_index_irq_source].m_sADQIRQInfo.m_uiSupportsExternal) != 0x00000000 )
                {
                    printf( "Only the following interrupts are available: 0x%08X - Press any key to continue\n\n", arr_irq_sources[ui_index_irq_source].m_sADQIRQInfo.m_uiSupportsExternal);

                    _getch();

                    continue;
                }

                break;
            }

            if(b_terminate)
            {
                continue;
            }

            while(true)
            {
                printf( "Enabled interrupts - Falling Edge (RETURN to exit): 0x");

                char sz_response[32];

                gets_s(&sz_response[0], 32);

                printf("\n\n");

                if(sz_response[0] == 0)
                {
                    b_terminate = true;
                        
                    break;
                }

                if( sscanf_s(&sz_response[0], "%X", &ui_external_falling_edge) != 1)
                {
                    continue;
                }

                if( (ui_external_falling_edge & ~arr_irq_sources[ui_index_irq_source].m_sADQIRQInfo.m_uiSupportsExternal) != 0x00000000 )
                {
                    printf( "Only the following interrupts are available: 0x%08X - Press any key to continue\n\n", arr_irq_sources[ui_index_irq_source].m_sADQIRQInfo.m_uiSupportsExternal);

                    _getch();

                    continue;
                }

                break;
            }

            if(b_terminate)
            {
                continue;
            }

            s_adq_irq_enable.m_uiExternalRisingEdge = ui_external_rising_edge;

            s_adq_irq_enable.m_uiExternalFallingEdge = ui_external_falling_edge;
        }

        //Disable the IRQ in case the program was previously closed with the [X] of the console window
        adqDisableIRQ(arr_irq_sources[ui_index_irq_source].m_uiBoardIndex,
                      arr_irq_sources[ui_index_irq_source].m_uiModuleIndex);

        printf("The structure SADQIRQEnable will have the following values:\n"
               "  m_uiSize                     = %u\n"
               "  m_uiID                       = 0x%08X\n"
               "  m_uiPortBitChangeRisingEdge  = 0x%08X\n"
               "  m_uiPortBitChangeFallingEdge = 0x%08X\n"
               "  m_bPortBitPatternCompare     = %S\n"
               "  m_uiPortBitPatternValue      = 0x%08X\n"
               "  m_uiExternalRisingEdge       = 0x%08X\n"
               "  m_uiExternalFallingEdge      = 0x%08X\n"
               "  m_uiPortStatusError          = 0x%08X\n"
               "  m_uiFaultRecovery            = 0x%08X\n"
               "  m_bResetIRQCount             = %S\n\n",
               s_adq_irq_enable.m_uiSize,
               s_adq_irq_enable.m_uiID,
               s_adq_irq_enable.m_uiPortBitChangeRisingEdge,
               s_adq_irq_enable.m_uiPortBitChangeFallingEdge,
               (s_adq_irq_enable.m_bPortBitPatternCompare ? L"Yes" : L"No"),
               s_adq_irq_enable.m_uiPortBitPatternValue,
               s_adq_irq_enable.m_uiExternalRisingEdge,
               s_adq_irq_enable.m_uiExternalFallingEdge,
               s_adq_irq_enable.m_uiPortStatusError,
               s_adq_irq_enable.m_uiFaultRecovery,
               (s_adq_irq_enable.m_bResetIRQCount ? L"Yes" : L"No"));

        printf("Press any key to enable the interrupt - Pressing a second key at any time will disable the interrupt again\n\n");

        _getch();

        ui_adq_error = adqEnableIRQ(arr_irq_sources[ui_index_irq_source].m_uiBoardIndex, 
                                    arr_irq_sources[ui_index_irq_source].m_uiModuleIndex, 
                                    s_adq_irq_enable, 
                                    adqIRQCallback, 
                                    SADQIRQStatus_ID, 
                                    NULL                                                    );

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("adqEnableIRQ returned ERROR 0x%08X - Press any key to continue\n", ui_adq_error);

            _getch();

            continue;
        }

        printf("Interrupt enabled - Press any key to disable the interrupt\n\n");

        _getch();

        ui_adq_error = adqDisableIRQ(arr_irq_sources[ui_index_irq_source].m_uiBoardIndex, 
                                     arr_irq_sources[ui_index_irq_source].m_uiModuleIndex);

        if (ui_adq_error == ADQ_ERROR_NONE)
        {
            printf("Interrupt disabled successfully. Press any key.\n\n");
        }

        // Remove the key stroke that caused kbhit to return 'true'.
        
        _getch();

        printf("Press any key to continue\n\n");

        _getch();
    }

    delete []arr_irq_sources;

    adqClose();

    printf("\n\n");

    printf("Press any key to terminate.\n");

    _getch();

    return 0;
}

unsigned int __stdcall adqIRQCallback(unsigned int uiBoardIndex, unsigned int uiModuleIndex, SADQIRQStatus* psIRQStatus, unsigned int uiError, void* pvUserContext)
{
    if (uiError != ADQ_ERROR_NONE)
    {
        printf("  adqIRQCallback received ERROR: 0x%08X\n", uiError);
    }
    else
    {
        printf("IRQ Count: %u\n", psIRQStatus->m_uiIRQCount);

        if (psIRQStatus->m_uiPortBitChangeRisingEdge != 0x00000000)
        {
            printf("  Rising edge: 0x%08X\n", psIRQStatus->m_uiPortBitChangeRisingEdge);
        }

        if (psIRQStatus->m_uiPortBitChangeFallingEdge != 0x00000000)
        {
            printf("  Falling edge: 0x%08X\n", psIRQStatus->m_uiPortBitChangeFallingEdge);
        }

        if (psIRQStatus->m_bPortBitPatternCompare)
        {
            printf("  Bit pattern compare\n");
        }

        if (psIRQStatus->m_uiExternalRisingEdge != 0x00000000)
        {
            printf("  External Rising edge: 0x%08X\n", psIRQStatus->m_uiExternalRisingEdge);
        }

        if (psIRQStatus->m_uiExternalFallingEdge != 0x00000000)
        {
            printf("  External Falling edge: 0x%08X\n", psIRQStatus->m_uiExternalFallingEdge);
        }

        if (psIRQStatus->m_uiPortStatusError != 0x00000000)
        {
            printf("  Port status error: 0x%08X\n", psIRQStatus->m_uiPortStatusError);
        }

        if (psIRQStatus->m_uiFaultRecovery != 0x00000000)
        {
            printf("  Fault recovery: 0x%08X\n", psIRQStatus->m_uiFaultRecovery);
        }

        printf("  Trigger value: 0x%08X\n\n", psIRQStatus->m_uiTriggerValue);
    }
    
    return uiError;
}

