//-----------------------------------------------------------------------------
// adqUtility.h - function prototypes for ALLDAQ Utility library
//      Copyright (c) ALLDAQ, a division of ALLNET GmbH
//      https://www.alldaq.com
//
// Purpose:
//      This file has defines for function prototypes, constants, and error codes
//      used by the ALLDAQ Utility library
//      - 32bit: adqUtility.dll
//      - 64bit: adqUtility_64.dll
//-----------------------------------------------------------------------------

#ifndef ADQUTILITYDLL_H
#define ADQUTILITYDLL_H

/*! \name	Constants for error codes - The return value from all utility functions

*/

//@{

const unsigned int ADQ_UTILITY_ERROR_NONE                       = 0;

const unsigned int ADQ_UTILITY_ERROR_DIGITAL_LIMITS_INVALID     = 1;

const unsigned int ADQ_UTILITY_ERROR_INVALID_PARAMETER_1        = 100;

const unsigned int ADQ_UTILITY_ERROR_INVALID_PARAMETER_2        = 101;

//@}


/*! \name   Constants for the parameter uiWindowFunction passed to the function adqUtilityFrequencySpectrum and adqUtilityFrequencySpectrumCoherentSampling

*/

//@{

const unsigned int  FFT_WINDOW_FUNCTION_RECTANGULAR             = 0;

const unsigned int  FFT_WINDOW_FUNCTION_HANN                    = 1;

const unsigned int  FFT_WINDOW_FUNCTION_HAMMING                 = 2;

const unsigned int  FFT_WINDOW_FUNCTION_BARTLETT                = 3;

const unsigned int  FFT_WINDOW_FUNCTION_FLAT_TOP                = 4;

const unsigned int  FFT_WINDOW_FUNCTION_HARRIS_NUTALL           = 5;

const unsigned int  FFT_WINDOW_FUNCTION_BLACKMAN_HARRIS_7_TERM  = 6;

//@}


#ifdef __cplusplus

extern "C"
{

#endif

/*! \name	Utility routines

*/

//@{


unsigned int __stdcall adqUtilityCalculateRMSValue(unsigned int* puiBuffer, unsigned int uiOffset, unsigned int uiStep, unsigned int uiNumberOfValues,
                                                        unsigned int uiDigitalMin, unsigned int uiDigitalMax, double dPhysicalMin, double dPhysicalMax, double& RMSValue);

/*!

    adqUtilityCalculateMinMaxAverageAndP2PValue below calculates the minimum and maximum values of the signal in physical units.
    dP2PValue is simply the difference of these two values,

*/

unsigned int __stdcall adqUtilityCalculateMinMaxAverageAndP2PValue(unsigned int* puiBuffer, unsigned int uiOffset, unsigned int uiStep, unsigned int uiNumberOfValues,
                                                                        unsigned int uiDigitalMin, unsigned int uiDigitalMax, double dPhysicalMin, double dPhysicalMax, double& dMinValue, double& dMaxValue, double& dAverageValue, double& dP2PValue);

unsigned int __stdcall adqUtilityFrequencySpectrum(double dSamplingRateHz, unsigned int* puiBuffer, unsigned int uiOffset, unsigned int uiStep, unsigned int uiNumberOfValues,
                                                                                    unsigned int uiDigitalMin, unsigned int uiDigitalMax, double dPhysicalMin, double dPhysicalMax,
                                                                                                double* pdFrequencySpectrum, double& dFrequencyStep, unsigned int uiFrequencySpectrumSize, unsigned int uiWindowFunction, double dOverlap);

unsigned int __stdcall adqUtilityFrequencySpectrumCoherentSampling(double dSamplingRateHz, double dSinusFrequencyHz,
                                                                        unsigned int* puiBuffer, unsigned int uiOffset, unsigned int uiStep, unsigned int uiNumberOfValues,
                                                                                        unsigned int uiDigitalMin, unsigned int uiDigitalMax, double dPhysicalMin, double dPhysicalMax,
                                                                                                 double* pdFrequencySpectrum, double& dFrequencyStep, unsigned int uiFrequencySpectrumSize, unsigned int uiWindowFunction, double dOverlap);

unsigned int __stdcall adqUtilityCalculateCharacteristicValuesFromFrequencySpectrum(double* pdFrequencySpectrum, double dFrequencyStep, unsigned int uiFrequencySpectrumSize,
                                                                                                                            double dFrequencyIgnoreBelow, double dFrequencyIgnoreAbove,
                                                                                                                                    double& dMainFrequencyHz, double& dProportionMainFrequency, double& dTHD, double& dTHDrPlusNoise, double& dTHDfPlusNoise, double& dRMSValue);

unsigned int __stdcall adqUtilityFilterFIR(unsigned int* puiBuffer, unsigned int uiOffset, unsigned int uiStep, unsigned int uiNumberOfValues,
                                                                           unsigned int uiDigitalMin, unsigned int uiDigitalMax, double dPhysicalMin, double dPhysicalMax,
                                                                                                                                        double* pdFIRCoefficients, unsigned int uiFIROrder);

unsigned int __stdcall adqUtilityFilterFIRGenerateHighPassCoefficients(double dSamplingRateHz, double dFilterOutToHz, double* pdFilterCoefficients, unsigned int uiFilterOrder);

unsigned int __stdcall adqUtilityFilterFIRGenerateLowPassCoefficients(double dSamplingRateHz, double dFilterOutFromHz, double* pdFilterCoefficients, unsigned int uiFilterOrder);

unsigned int __stdcall adqUtilityFilterFIRGenerateBandPassCoefficients(double dSamplingRateHz, double dFilterOutToHz, double dFilterOutFromHz, double* pdFilterCoefficients, unsigned int uiFilterOrder);

unsigned int __stdcall adqUtilityFilterFIRGenerateBandStopCoefficients(double dSamplingRateHz, double dFilterOutFromHz, double dFilterOutToHz, double* pdFilterCoefficients, unsigned int uiFilterOrder);

unsigned int __stdcall adqUtilityFilterFIRGenerateAWeightedCoefficients(double dSamplingRateHz,  double* pdFilterCoefficients, unsigned int uiFilterOrder);

unsigned int __stdcall adqUtilityFilterFIRGenerateITUR468WeightedCoefficients(double dSamplingRateHz,  double* pdFilterCoefficients, unsigned int uiFilterOrder);

unsigned int _stdcall adqUtilityConvertLinearTodBu(double dReferenceValue, double dLinearValue, double& ddBuValue);

//@}

#ifdef __cplusplus

}

#endif

#endif




