//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the adqWriteContinuousCallback project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

#pragma once

#include "..\\..\\..\\adqDevice.h"

#include "AoRange.h"
#include "WaveForm.h"

#include <string>
#include <vector>
#include <memory>

class AoModule
{
public:
    AoModule(const std::wstring &boardName, unsigned int boardIndex, unsigned int moduleIndex, 
        const SADQGeneralModuleInfo &generalInfo, const SADQAOModuleInfo &aoModuleInfo);
    virtual ~AoModule();

    std::wstring GetModuleName();
    double GetMaxFrequency();
    void ClearErrorInfo();
    std::vector<std::wstring> GetErrorInfo();
    std::shared_ptr<AoRange> GetSelectedRange();
    unsigned int SetFrequency(double nominalFrequency);
    double GetActualOutputFrequency();
    void SetSignalGenerator(std::shared_ptr<ISignalGenerator> &signalGenerator);
    std::shared_ptr<ISignalGenerator> GetSelectedSignalGenerator();
    void SetNumberOfValuesPerSignalPeriod(int numValuesPerPeriod);
    double GetSignalFrequency();
    void SetSignalAmplitude(double signalAmplitude);
    double GetSignalAmplitude();
    unsigned int StartOutput();
    unsigned int WriteValues(unsigned int numberOfValuesAcceptable, unsigned int* values);
    unsigned int StopOutput();
    void DeleteValueBuffers();
    unsigned int m_AdqError;
private:
    unsigned int m_boardIndex;
    unsigned int m_moduleIndex;
    double m_maxFrequency;
    double m_actualOutputFrequency;
    double m_signalFrequency;
    double m_signalAmplitude;
    unsigned int* m_outputValueBuffer;
    double* m_physicalValueBuffer;
    unsigned int m_outputBufferLength;
    unsigned int m_valueOffset;
    int m_numValuesPerPeriod;
    unsigned int m_notifyAfterValues;

    std::wstring m_moduleName;
    std::vector<std::shared_ptr<AoRange>> m_ranges;
    std::shared_ptr<AoRange> m_selectedRange;
    std::unique_ptr<WaveForm> m_waveForm;
    std::shared_ptr<ISignalGenerator> m_selectedSignalGenerator;
    std::vector<std::wstring> m_errorInfo;
    void AddErrorInfo(const wchar_t * functionName, unsigned int adqError);
    void InitOutputBuffer();
    void SetNotifyAfterValues();
    unsigned int EvaluateOutputBufferLength(unsigned int notifyAfterValues);
    void InitWaveForm();
    void CalculateValueBuffers();
    void CalculateDigitalValueBuffer();
    unsigned int StartContinuousOutput();
    void AddRanges(const SADQAOModuleInfo &aoModuleInfo);
    void BuildModuleName(const std::wstring &boardName);
};

