//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the adqWriteContinuousWithoutCallback project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

#include "stdafx.h"
#include "AoRange.h"
#include "AdqCommon.h"

#include <sstream>

AoRange::AoRange(unsigned int rangeIndex, unsigned int digitalMin, unsigned int digitalMax,
    unsigned int physicalUnit, double physicalMin, double physicalMax)
    : m_rangeIndex(rangeIndex),
    m_digitalMin(digitalMin),
    m_digitalMax(digitalMax),
    m_physicalMin(physicalMin),
    m_physicalMax(physicalMax)
{
    m_unitSymbol = ALLDAQ::GetPhysicalUnitSymbol(physicalUnit);
    m_physicalResolution = (physicalMax - physicalMin) / (digitalMax - digitalMin);
    m_digitalOffset = digitalMin - physicalMin / m_physicalResolution;

    SetRangeText();
}

AoRange::~AoRange()
{
}

unsigned int AoRange::PhysicalToDigital(double physicalValue)
{
    return (unsigned int)(std::round(physicalValue / m_physicalResolution) + m_digitalOffset);
}

double AoRange::GetPhysicalMax()
{
    return m_physicalMax;
}

double AoRange::GetPhysicalMin()
{
    return m_physicalMin;
}

std::wstring AoRange::GetRangeText()
{
    return m_rangeText;
}

std::wstring AoRange::GetUnitSymbol()
{
    return m_unitSymbol;
}

void AoRange::SetRangeText()
{
    std::wstringstream wss;
    wss << ALLDAQ::FormatDoubleValue(m_physicalMin, 2)
        << m_unitSymbol
        << L"..."
        << ALLDAQ::FormatDoubleValue(m_physicalMax, 2)
        << m_unitSymbol;
    m_rangeText = wss.str();
}

