//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the adqWriteContinuousWithoutCallback project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

#include "stdafx.h"
#include "WaveForm.h"

#include <cmath>

WaveForm::WaveForm()
    : m_outputFrequency(0.0),
    m_physicalValues(nullptr),
    m_numberOfValues(0),
    m_rangePhysicalMin(0.0),
    m_rangePhysicalMax(0.0)
{
}

WaveForm::~WaveForm()
{
}

void WaveForm::InitWaveForm(double outputFrequency, double* physicalValues, unsigned int numberOfValues, double rangePhysicalMin, double rangePhysicalMax)
{
    m_outputFrequency = outputFrequency;
    m_physicalValues = physicalValues;
    m_numberOfValues = numberOfValues;
    m_rangePhysicalMin = rangePhysicalMin;
    m_rangePhysicalMax = rangePhysicalMax;

    for (unsigned int i = 0; i < numberOfValues; i++)
    {
        physicalValues[i] = 0.0;
    }
}

void WaveForm::ApplySignalGenerator(std::shared_ptr<ISignalGenerator> &signalGenerator, double signalFrequency, double signalPhysicalMax)
{
    for (unsigned int i = 0; i < m_numberOfValues; i++)
    {
        double time = i / m_outputFrequency;

        double timeOffset = time * signalFrequency;
        double cycleTime = timeOffset - floor(timeOffset);
        double currValue = m_physicalValues[i] + signalGenerator->GetValue(cycleTime, signalPhysicalMax);

        if (currValue < m_rangePhysicalMin)
        {
            currValue = m_rangePhysicalMin;
        }
        else if (currValue > m_rangePhysicalMax)
        {
            currValue = m_rangePhysicalMax;
        }

        m_physicalValues[i] = currValue;
    }
}