//-----------------------------------------------------------------------------
// adqDeviceConstants.h - Constants for ALLDAQ driver system API
//      Copyright (c) ALLDAQ, a division of ALLNET GmbH
//      https://www.alldaq.com
//
// Purpose:
//      This file has defines for constants
//      used by ALLDAQ measurement boards
//-----------------------------------------------------------------------------

#ifndef ADQDEVICECONSTANTS_H
#define ADQDEVICECONSTANTS_H

/*! \name   Constants used for the first four parameters of the routine adqQuery
       
*/

//@{

const unsigned int ADQ_QUERY_UNUSED_PARAMETER           = 0xFFFFFFFF;       //!<    Used for any parameter in adqQuery which is not required in a particular context


//! System constants for the parameter 'uiID' in adqQuery. Used with ADQ_SYSTEM_HANDLE as the first parameter. The second parameter 'uiIndexModule' should always be ADQ_QUERY_UNUSED_PARAMETER.

const unsigned int ADQ_QUERY_SYSTEM_NUMBER_OF_BOARDS    = 0x00000001;       //!<    'uiSubId' should be ADQ_QUERY_UNUSED_PARAMETER. 'pucBuffer' should be a pointer to an unsigned integer, which, when the function returns, will contain the number of boards in the system.

const unsigned int ADQ_QUERY_SYSTEM_BOARD_HANDLE        = 0x00000002;       //!<    'uiSubID' is the board index. 'pucBuffer' 'pucBuffer' should be a pointer to an ADQHANDLE, which, when the function returns, will contain the board handle which should be used in all other API functions to access the board.


//!< Board constants for the parameter 'uiID' in adqQuery. Used with the board handle as the first parameter. The second parameter 'uiIndexModule' should always be ADQ_QUERY_UNUSED_PARAMETER.

const unsigned int ADQ_QUERY_BOARD_NUMBER_OF_MODULES    = 0x00010001;       //!<    'uiSubID' should be ADQ_QUERY_UNUSED_PARAMETER. 'pucBuffer' should be a pointer to an unsigned integer, which, when the function returns, will contain the number of modules on the board.

const unsigned int ADQ_QUERY_BOARD_INFO                 = 0x00010002;       //!<    'uiSubID' should be ADQ_QUERY_UNUSED_PARAMETER. 'pucBuffer' should be a pointer to an SADQBoardInfo structure, which, when the function returns, will contain information about the board. /note This information includes the number of modules on the board.


//!< Module constants for the parameter 'uiID' in adqQuery. Used with the board handle as the first parameter. The second parameter 'uiIndexModule' is the 0 based module index.

const unsigned int ADQ_QUERY_MODULE_TYPE                = 0x00020001;       //!<    'uiSubID' should be ADQ_QUERY_UNUSED_PARAMETER. 'pucBuffer' should be a pointer to an unsigned integer, which, when the function returns, will contain the module type. This is one of the constants ADQ_MODULE_TYPE_... below.

const unsigned int ADQ_QUERY_MODULE_INFO                = 0x00020002;       //!<    'uiSubID' should be ADQ_QUERY_UNUSED_PARAMETER. 'pucBuffer' should be a pointer to the corresponding SADQXXXModuleInfo structure below..., which when the function returns contains the modul information

//@}


/*! \name   System constants for the variable m_uiModuleType in the structure SADQGeneralModuleInfo below
       
*/

//@{

const unsigned int  ADQ_MODULE_TYPE_DI                  = 0x00030000;       //!<    Digital input 

const unsigned int  ADQ_MODULE_TYPE_DO                  = 0x00030001;       //!<    Digital output 

const unsigned int  ADQ_MODULE_TYPE_DIO                 = 0x00030002;       //!<    Digital input / output 

const unsigned int  ADQ_MODULE_TYPE_CNTR                = 0x00030003;       //!<    Counter 

const unsigned int  ADQ_MODULE_TYPE_EXTINT              = 0x00030004;       //!<    External interrupt 

const unsigned int  ADQ_MODULE_TYPE_AI                  = 0x00030005;       //!<    Analog input

const unsigned int  ADQ_MODULE_TYPE_AO                  = 0x00030006;       //!<    Analog output

const unsigned int  ADQ_MODULE_TYPE_I2C                 = 0x00030007;       //!<    I2C

const unsigned int  ADQ_MODULE_TYPE_PWM                 = 0x00030008;       //!<    PWM

const unsigned int  ADQ_MODULE_TYPE_FIN                 = 0x00030009;       //!<    FIN

const unsigned int  ADQ_MODULE_TYPE_I2SO                = 0x0003000A;       //!<    I2S output

//@}

/*! \name   System constants for the parameter uiDirection passed to adqDIOSetDirection and returned from adqDIOGetDirection
       
*/

//@{

const unsigned int  ADQ_DIO_INPUT                       = 0x00040000;       //!<    Digital input 

const unsigned int  ADQ_DIO_OUTPUT                      = 0x00040001;       //!<    Digital output 

//@}

/*! \name   System constants for the parameter uiClockSource passed to the function adqCNTRConfig and for 
            the variable m_uiClockSources in the structure SADQCNTRModuleInfo below.
            .

            \note   The parameter uiClockSource passed to adqCNTRConfig should be one of the constants below.
                    If the counter module only supports one clock source, then this can also be selected by
                    passing the constant ADQ_CNTR_CLOCK_SOURCE_DEFAULT.
                    In the structure SADQCNTRModuleInfo the variable m_uiClockSources is a combination (OR'ed)  
                    of any of the constants below except 0x00050000.
       
*/

//@{

const unsigned int  ADQ_CNTR_CLOCK_SOURCE_DEFAULT           = 0x00050000;       //!<    Clock source default - Can be passed to adqCNTRConfig if the module only supports one clock source 

const unsigned int  ADQ_CNTR_CLOCK_SOURCE_EXTERNAL          = 0x00050001;       //!<    Clock source external 

const unsigned int  ADQ_CNTR_CLOCK_SOURCE_INTERNAL_10_MHZ   = 0x00050002;       //!<    Clock source internal 10 Megahertz 

const unsigned int  ADQ_CNTR_CLOCK_SOURCE_INTERNAL_66_MHZ   = 0x00050004;       //!<    Clock source internal 66 Megahertz 

//@}

/*! \name   System constants for the parameter uiCntrMode passed to the function adqCNTRConfig
       
*/

//@{

const unsigned int  ADQ_CNTR_8254_MODE_0                = 0x00060000;       //!<    Mode 0: Change state at zero 

const unsigned int  ADQ_CNTR_8254_MODE_1                = 0x00060001;       //!<    Mode 1: Retriggerable "One Shot" 

const unsigned int  ADQ_CNTR_8254_MODE_2                = 0x00060002;       //!<    Mode 2: Asymmetric divider 

const unsigned int  ADQ_CNTR_8254_MODE_3                = 0x00060003;       //!<    Mode 3: Symmetric divider 

const unsigned int  ADQ_CNTR_8254_MODE_4                = 0x00060004;       //!<    Mode 4: Counter start by software trigger 

const unsigned int  ADQ_CNTR_8254_MODE_5                = 0x00060005;       //!<    Mode 5: Counter start by hardware trigger 

//@}

/*! \name   System constants for the parameter uiCalibrationID returned by adqAnalogGetSelectedCalibration and passed to adqAnalogSelectCalibration
       
*/

//@{

const unsigned int  ADQ_CALIBRATION_FACTORY             = 0x00070000;      

const unsigned int  ADQ_CALIBRATION_USER_1              = 0x00070001;      

const unsigned int  ADQ_CALIBRATION_USER_2              = 0x00070002;      

const unsigned int  ADQ_CALIBRATION_USER_3              = 0x00070003;      

const unsigned int  ADQ_CALIBRATION_USER_4              = 0x00070004;      

const unsigned int  ADQ_CALIBRATION_NONE                = 0x0007FFFF;      

//@}

/*! \name   System constants for the variable m_uiTTLVoltageLevel returned in the structure SADQBoardInfo below. These constants are ORed together.
            One of these constants should also passed to the routine adqSetBoardTTLVoltage in the parameter 'uiVoltage'
*/

//@{

const unsigned int  ADQ_TTL_VOLTAGE_LEVEL_3_3_VOLTS    = 0x00000001;   //!< 3.3 Volts 

const unsigned int  ADQ_TTL_VOLTAGE_LEVEL_5_0_VOLTS    = 0x00000002;   //!< 5.0 Volts      

//@}
/*! \name   System constants for the variable m_uiClockType in the structure SADQGeneralModuleInfo below
       
*/

//@{

const unsigned int  ADQ_CLOCK_TYPE_NONE                 = 0x00080000;      

const unsigned int  ADQ_CLOCK_TYPE_SOFTWARE             = 0x00080001;      

const unsigned int  ADQ_CLOCK_TYPE_HARDWARE             = 0x00080002;

//@}

/*! \name   System constants for the variable m_uiType in the structure SADQCNTRModuleInfo below
       
*/

//@{

const unsigned int  ADQ_CNTR_TYPE_8254                  = 0x00090000;       //!<    Counter type 8254 

const unsigned int  ADQ_CNTR_TYPE_COUNT_DOWN_32_BIT     = 0x00090001;       //!<    Counter type Count Down 32-Bit 

//@}

/*! \name   System constants for the variable m_uiPhysicalUnit in the structure SADQAIRange below
       
*/

//@{

const unsigned int  ADQ_PHYSICAL_UNIT_VOLT              = 0x000A0000;      

const unsigned int  ADQ_PHYSICAL_UNIT_AMPERE            = 0x000A0001;      

const unsigned int  ADQ_PHYSICAL_UNIT_OHM               = 0x000A0002;

//@}

/*! \name   System constants for the variable m_uiBusType in the structure SADQBoardInfo below
       
*/

//@{

const unsigned int  ADQ_BUS_TYPE_PCI                    = 0x000B0000;      

const unsigned int  ADQ_BUS_TYPE_USB                    = 0x000B0001;      

//@}

/*! \name   System constants for the parameter 'uiReasonForCall' passed by the system to the callback routines ADQ_..._CALLBACK below.
       
*/

//@{

const unsigned int  ADQ_CALLED_FOR_START                = 0x00000001;      

const unsigned int  ADQ_CALLED_FOR_DATA                 = 0x00000002;      

const unsigned int  ADQ_CALLED_FOR_STOP                 = 0x00000004;

//@}


/*! \name   System constants for using with adqWaitIRQ 
       
*/

//@{

const unsigned int ADQ_TIMEOUT_INFINITE = 0; 

//@}

/*! \name   System constants for using with continuous functions
       
*/

//@{

const unsigned int ADQ_NUMBER_UNKNOWN = 0; 

//@}

/*! \name   System constants for using with adqEnableHWTrigger
       
*/

//@{

const unsigned int ADQ_HW_START_TRIGGER_TIMEOUT_INFINITE = 0xFFFFFFFF; 

//@}

/*! \name   System constants for using with adqPWMEnable

*/

//@{

const unsigned int ADQ_PWM_TIMEOUT_INFINITE = 0xFFFFFFFF;

//@}

/*! \name   System constants for using with adqFINRead

*/

//@{

const unsigned int ADQ_FIN_TIMEOUT_INFINITE = 0xFFFFFFFF;

//@}


//!< Structures 

#pragma pack(push, 4)

/*! \name   System information structure, passed to adqGetSystemInfo
       
*/

//@{

struct SADQSystemInfo
{
    unsigned int    m_uiSize;

    unsigned int    m_uiID;                 //!< Use SADQSystemInfo_ID below 

    unsigned int    m_uiDLLVersion;

    unsigned int    m_uiNumberOfBoards;
};

const unsigned int SADQSystemInfo_ID                    = 0x00010000;

//@}

/*! \name   Board information structure, passed to adqGetBoardInfo
       
*/

//@{

struct SADQBoardInfo
{
    unsigned int    m_uiSize;

    unsigned int    m_uiID;                                 //!< Use SADQBoardInfo_ID below 

    unsigned short  m_usVendorID;

    unsigned short  m_usDeviceID;

    unsigned int    m_uiSerialNumber;

    wchar_t         m_wszBoardName[64];

    wchar_t         m_wszBoardDescription[256];

    wchar_t         m_wszDriverName[16];

    unsigned int    m_uiDriverVersion;

    unsigned int    m_uiTTLVoltageLevel;                    //!< A bit-coded combination of the constants ADQ_TTL_VOLTAGE_LEVEL_... above. 0 --> Not applicable

    unsigned int    m_uiBusType;                            //!< One of the constants ADQ_BUS_TYPE_... above.

    bool            m_bRemote;                              //!< false --> Installed on the local system, true --> installed on a remote system

    wchar_t         m_wszRemoteIPAddressOrHostName[256];    //!< Used if m_bRemote above is true

    unsigned int    m_uiFirmwareVersion;

    unsigned int    m_uiNumberOfModules;
};

const unsigned int SADQBoardInfo_ID                    = 0x00020001;

//@}


/*! \name   Board PCI information structure, passed to adqGetBoardPCIInfo
       
*/

//@{

struct SADQBoardPCIInfo
{
    unsigned int    m_uiSize;

    unsigned int    m_uiID;                                 //!< Use SADQBoardPCIInfo_ID below 

    unsigned int    m_uiBusNumber;

    unsigned int    m_uiDeviceNumber;

    unsigned int    m_uiFunctionNumber;
};

const unsigned int SADQBoardPCIInfo_ID                    = 0x000D0000;

//@}

/*! \name   Board synchron start information structure, passed to adqGetBoardSynchronStartInfo
       
*/

//@{

struct SADQBoardSynchronStartInfo
{
    unsigned int    m_uiSize;

    unsigned int    m_uiID;                                 //!< Use SADQBoardSynchronStartInfo_ID below 

    unsigned int    m_uiNumberOfGroups;

    unsigned int    m_uiSizeOfGroup;

    unsigned int    m_uiModuleMap;  
};

const unsigned int SADQBoardSynchronStartInfo_ID       = 0x00300000;

//@}

/*! \name   Module information structures, passed to adqGetModuleInfo.
       
*/

//@{

    //! General module information - structure passed as the third parameter to the function adqGetModuleInfo.

struct SADQGeneralModuleInfo
{
    unsigned int    m_uiSize;

    unsigned int    m_uiID;                             //!< Use SADQGeneralModuleInfo_ID below 

    wchar_t         m_wszModuleDescription[1024];

    unsigned int    m_uiModuleType;                     //!< One of the constants ADQ_MODULE_TYPE_... above.

    bool            m_bIsIRQSource;

    unsigned int    m_uiClockType;                      //!< One of the constants ADQ_CLOCK_TYPE_... above.

    double          m_dClockBaseFrequencyHz;

    unsigned int    m_uiClockMinimumDivider;

    unsigned int    m_uiInternalDataBufferSizeValues;       
};

const unsigned int SADQGeneralModuleInfo_ID             = 0x00030001;

/*! \name   Structures for adq trigger routines
       
*/

//@{


//! Trigger information structure passed as the third parameter to the function adqGetTriggerInfo.

struct SADQHWTriggerInfo
{
    unsigned int    m_uiSize;

    unsigned int    m_uiID;                         //!< Use SADQHWTriggerInfo_ID below 

    unsigned int    m_uiHWTriggerStartSource;

    unsigned int    m_uiHWTriggerStopSource;

    bool            m_bSupportsChannelListPerTrigger;
};

const unsigned int SADQHWTriggerInfo_ID                   = 0x000B0000;

//! Trigger enable structure passed as the third parameter to the function adqEnableHWTrigger.

struct SADQHWTriggerEnable
{
    unsigned int    m_uiSize;

    unsigned int    m_uiID;                         //!< Use SADQHWTriggerEnable_ID below 

    unsigned int    m_uiHWTriggerStartRisingEdge;

    unsigned int    m_uiHWTriggerStartFallingEdge;

    unsigned int    m_uiHWTriggerStopRisingEdge;

    unsigned int    m_uiHWTriggerStopFallingEdge;

    bool            m_bChannelListPerTrigger;

    bool            m_bAutomaticRearm;
};

const unsigned int SADQHWTriggerEnable_ID                 = 0x000C0000;

//@}

/*! \name   Module information for particular module types - structure passed as the fifth parameter to the function adqGetModuleInfo.

*/

//@{

//! Module information for DI modules.

const unsigned int ADQ_MAX_DI_FILTERS   = 4;

struct SADQDIModuleInfo
{
    unsigned int        m_uiSize;

    unsigned int        m_uiID;                         //!< Use SADQDIOModuleInfo_ID below 

    unsigned int        m_uiPortWidthBits;

    bool                m_bIsOptoIsolated;

    bool                m_bIsTTL;

    unsigned int        m_uiNumberOfFilters;            //!< Number of filters

    unsigned __int64    m_arrui64FilterTimePicoseconds[ADQ_MAX_DI_FILTERS];
};

const unsigned int SADQDIModuleInfo_ID                 = 0x00040001;

//! Module information for DO and DIO modules.

struct SADQDIOModuleInfo
{
    unsigned int    m_uiSize;

    unsigned int    m_uiID;                 //!< Use SADQDIOModuleInfo_ID below 

    unsigned int    m_uiPortWidthBits;

    bool            m_bIsOptoIsolated;

    bool            m_bIsTTL;
};

const unsigned int SADQDIOModuleInfo_ID                 = 0x00040000;

typedef struct SADQDIOModuleInfo SADQDOModuleInfo;

const unsigned int SADQDOModuleInfo_ID = SADQDIOModuleInfo_ID;

//! Module information for CNTR modules.

struct SADQCNTRModuleInfo
{
    unsigned int    m_uiSize;

    unsigned int    m_uiID;                 //!< Use SADQCNTRModuleInfo_ID below 

    unsigned int    m_uiType;

    unsigned int    m_uiWidthBits;

    unsigned int    m_uiClockSources;
};

const unsigned int SADQCNTRModuleInfo_ID                = 0x00050000;

//! Module information for AI modules.

struct SADQAIRange
{
    unsigned int    m_uiDigitalMin;

    unsigned int    m_uiDigitalMax;

    unsigned int    m_uiPhysicalUnit;   //!< One of ADQ_PHYSICAL_UNIT... above

    double          m_dPhysicalMin;

    double          m_dPhysicalMax;
};

const unsigned int  ADQ_MAX_AI_RANGES   = 8;

struct SADQAIModuleInfo
{
    unsigned int    m_uiSize;

    unsigned int    m_uiID;                 //!< Use SADQAIModuleInfo_ID below 

    unsigned int    m_uiNumberOfChannels;

    bool            m_bSharedDataRate;

    bool            m_bRangeInfoPerChannel;

    unsigned int    m_uiNumberOfRanges;

    SADQAIRange     m_arrRanges[ADQ_MAX_AI_RANGES];
};

const unsigned int SADQAIModuleInfo_ID                  = 0x00060000;

//! Module information for AO modules.

struct SADQAORange
{
    unsigned int    m_uiDigitalMin;

    unsigned int    m_uiDigitalMax;

    unsigned int    m_uiPhysicalUnit;   //!< One of ADQ_PHYSICAL_UNIT... above

    double          m_dPhysicalMin;

    double          m_dPhysicalMax;
};

const unsigned int  ADQ_MAX_AO_RANGES   = 8;

struct SADQAOModuleInfo
{
    unsigned int    m_uiSize;

    unsigned int    m_uiID;                 //!< Use SADQAOModuleInfo_ID below 

    unsigned int    m_uiNumberOfRanges;

    SADQAORange     m_arrRanges[ADQ_MAX_AO_RANGES];
};

const unsigned int SADQAOModuleInfo_ID                  = 0x000A0000;

//! Module information for I2C modules.

struct SADQI2CModuleInfo
{
    unsigned int    m_uiSize;

    unsigned int    m_uiID;                 //!< Use SADQI2CModuleInfo_ID below 

    unsigned int    m_uiSpeedBitsPerSecond;

    bool            m_bSupports7BitAddresses;

    bool            m_bSupports10BitAddresses;
};

const unsigned int SADQI2CModuleInfo_ID                = 0x000E0000;

//! Module information for PWM modules.

struct SADQPWMModuleInfo
{
    unsigned int    m_uiSize;

    unsigned int    m_uiID;                 //!< Use SADQPWMModuleInfo_ID below 

    double          m_dBaseFrequencyHz;

    unsigned int    m_uiMinimumDivider;
};

const unsigned int SADQPWMModuleInfo_ID                = 0x000F0000;

//@}

/*! \name   Structures for adq IRQ routines
       
*/

//@{

//! IRQ Information - structure passed as the third parameter to the function adqGetIRQInfo.

struct SADQIRQInfo
{
    unsigned int    m_uiSize;

    unsigned int    m_uiID;                         //!< Use SADQIRQInfo_ID below 

    wchar_t         m_wszIRQDescription[1024];

    bool            m_bSupportsBitChange;           //!< The module is a port (DI, DO or DIO) and can generate an interrupt when one of its bits changes (rising edge, falling edge or both).

    bool            m_bSupportsBitPatternCompare;   //!< The module is a port (DI, DO or DIO) and can generate an interrupt when its bits match a user-defined pattern.

    unsigned int    m_uiSupportsExternal;           //!< The module can generate an interrupt when the state of certain external pins changes. Each bit which is set here represents a different pin. See m_wszIRQDescription or documentation for details of which pins are concerned.

    unsigned int    m_uiSupportsPortStatusError;    //!< The module can generate an interrupt when a hardware failure occured such as over temperature, short circuit, external power loss, ground disconnected etc. - see m_wszIRQDescription or documentation for board and module

    unsigned int    m_uiSupportsFaultRecovery;      //!< The module can generate an interrupt when a physical value (temperature, voltage etc. - see m_wszIRQDescription or documentation for board and module) doesn't persist any more.
};

const unsigned int SADQIRQInfo_ID                       = 0x00070001;

//! IRQ Configuration - structure passed as the third parameter to the function adqEnableIRQ.

struct SADQIRQEnable
{
    unsigned int    m_uiSize;

    unsigned int    m_uiID;                             //!< Use SADQIRQEnable_ID below 

    unsigned int    m_uiPortBitChangeRisingEdge;        //!< Set those bits to 1 which should trigger an interrupt on changing from 0 to 1. This and 'm_uiPortBitChangeFallingEdge' below should be set to 0x00000000 to disable the bit change interrupt.

    unsigned int    m_uiPortBitChangeFallingEdge;       //!< Set those bits to 1 which should trigger an interrupt on changing from 1 to 0. This and 'm_uiPortBitChangeRisingEdge' above should be set to 0x00000000 to disable the bit change interrupt. 

    bool            m_bPortBitPatternCompare;           //!< 'true' if an Interrupt should be generated when pattern in 'm_uiPortBitPatternValue' below is matched.

    unsigned int    m_uiPortBitPatternValue;            //!< If 'm_bPortBitPatternCompare' above is 'true' this is the bit pattern which should generate an interrupt when it is matched.

    unsigned int    m_uiExternalRisingEdge;             //!< A bit here should be set to '1' if a change from 0 to 1 on the corresponding external pin should cause an interrupt.

    unsigned int    m_uiExternalFallingEdge;            //!< A bit here should be set to '1' if a change from 1 to 0 on the corresponding external pin should cause an interrupt.

    unsigned int    m_uiPortStatusError;                //!< A bit here should be set to '1' if an Interrupt should be generated when the port detects an hardware failure (over temperature, short circuit, external power loss, ground disconnected etc.).

    unsigned int    m_uiFaultRecovery;                  //!< A bit here should be set to '1' if an Interrupt should be generated when the port status error does not persist any more.

    bool            m_bResetIRQCount;                   //!< 'true' if the interrupt count should be reset to zero on calling the function.
};

const unsigned int SADQIRQEnable_ID                       = 0x00080001;

//! IRQ Status - structure passed as the third parameter to the function adqWaitIRQ and passed as the third parameter in a ADQ_IRQ_CALLBACK function.

struct SADQIRQStatus
{
    unsigned int    m_uiSize;

    unsigned int    m_uiID;                             //!< Use SADQIRQStatus_ID below 

    unsigned int    m_uiIRQCount;                       //!< The total number of interrupts which have occurred since the count was last reset. 

    unsigned int    m_uiPortBitChangeRisingEdge;        //!< Those bits for which the interrupt is enabled are set to 1 if they have changed from 0 to 1 since the last time the IRQ callback was executed or since the last time the user called adqWaitIRQ.

    unsigned int    m_uiPortBitChangeFallingEdge;       //!< Those bits for which the interrupt is enabled are set to 1 if they have changed from 1 to 0 since the last time the IRQ callback was executed or since the last time the user called adqWaitIRQ.

    bool            m_bPortBitPatternCompare;           //!< 'true' if the interrupt is enabled and if a pattern match has occurred since the last time the IRQ callback was executed or since the last time the user called adqWaitIRQ.

    unsigned int    m_uiExternalRisingEdge;             //!< A bit here is '1' if the interrupt is enabled and if a change from 0 to 1 on the corresponding external pin has occurred since the last time the IRQ callback was executed or since the last time the user called adqWaitIRQ.

    unsigned int    m_uiExternalFallingEdge;            //!< A bit here is '1' if the interrupt is enabled and if a change from 1 to 0 on the corresponding external pin has occurred since the last time the IRQ callback was executed or since the last time the user called adqWaitIRQ.

    unsigned int    m_uiPortStatusError;                //!< A bit here is '1' if the interrupt is enabled and if a status error on the port has occurred since the last time the IRQ callback was executed or since the last time the user called adqWaitIRQ.

    unsigned int    m_uiFaultRecovery;                  //!< A bit here is '1' if the interrupt is enabled and if the error condition of 'm_uiPortStatusError' does not persist any more since the last time the IRQ callback was executed or since the last time the user called adqWaitIRQ.

    unsigned int    m_uiTriggerValue;                   //!< The value that was actually encountered on the module when the last interrupt occurred.
};

const unsigned int SADQIRQStatus_ID                       = 0x00090001;

//@}

/*! 
    AI channel - An array of these structures is passed as the third parameter to the function adqAISetChannelList.

        \note   The first element in the list represents the first channel (index 0) the second element represents the second channel (index 1) and so on.
                So if you want to acquire data from channels 0 and 7 you must create an array of at least 8 such structures in which 'm_bAcquireChannel' 
                is set to 'true' in the first (index 0) and eighth (index 7) elements of the array and set to 'false' in the other elements.
*/

struct SADQAIChannel
{
    bool m_bAcquireChannel;

    unsigned int m_uiRangeIndex;
};

/*! \name      Config information for particular counter types - structure passed as the fourth parameter to the function adqCNTRConfigEx.

*/

//@{

//! Module configuration for counter modules of type 8254 (ADQ_CNTR_TYPE_8254).

struct SADQConfigCNTR8254
{
    unsigned int        m_uiSize;

    unsigned int        m_uiID;                         //!< Use SADQConfigCNTR8254_ID below 

    unsigned int        m_uiClockSource;                //!< One of the constants ADQ_CNTR_CLOCK_SOURCE_EXTERNAL, ADQ_CNTR_CLOCK_SOURCE_INTERNAL_10_MHZ or ADQ_CNTR_CLOCK_SOURCE_INTERNAL_66_MHZ above

    unsigned int        m_uiCNTR8254Mode;               //!< One of the constants ADQ_CNTR_8254_MODE_... above

    unsigned int        m_uiStartValue;
};

const unsigned int SADQConfigCNTR8254_ID = 0x00100000;


//! Module configuration for counter modules of type count down 32-Bit (ADQ_CNTR_TYPE_COUNT_DOWN_32_BIT).

struct SADQConfigCNTRCountDown32Bit
{
    unsigned int        m_uiSize;

    unsigned int        m_uiID;                         //!< Use SADQConfigCNTRCountDown32Bit_ID below 

    unsigned int        m_uiClockSource;                //!< One of the constants ADQ_CNTR_CLOCK_SOURCE_EXTERNAL, ADQ_CNTR_CLOCK_SOURCE_INTERNAL_10_MHZ or ADQ_CNTR_CLOCK_SOURCE_INTERNAL_66_MHZ above

    unsigned int        m_uiStartValue;

    unsigned int        m_uiThresholdValue;             //!< Only required if the threshold interrupt is used, otherwise 0

    bool                m_bOneShot;                     //!< false --> reload start value and continue counting on reaching zero, true --> stop counting on reaching zero

    unsigned int        m_uiStrobeValue;                //!< Length of signal on the 'strobe' line after reaching 0. 0 --> NO strobe signal

    bool                m_bStrobeInvert;                //!< 0 --> Strobe signal NOT inverted, 1 --> Strobe signal inverted.

    bool                m_bHardwareGate;                //!< false --> Count down on every clock impulse , true --> Count down on clock impulse only if gate is open.

    bool                m_bHardwareTrigger;             //!< false --> Triggered by software, true --> Triggered by hardware signal 
};

const unsigned int SADQConfigCNTRCountDown32Bit_ID = 0x00200000;

//@}


/*! \name      Typedefs for function pointers used in the API.

*/

//@{

//! Typedef for the function pointer passed as the fourth parameter to adqEnableIRQ

typedef unsigned int (__stdcall *ADQ_IRQ_CALLBACK)(unsigned int uiBoardIndex, unsigned int uiModuleIndex, SADQIRQStatus* psIRQStatus, unsigned int uiError, void* pvUserContext);

//! Typedef for the function pointer passed as the fifth parameter to adqReadContinuousStart - The parameter 'uiReasonForCall' is a bit-coded combination of the constants ADQ_CALLED_FOR_... above

typedef unsigned int (__stdcall *ADQ_READ_CONTINUOUS_CALLBACK)(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiNumberOfValuesAvailable, unsigned int* puiValues, unsigned int uiError, void* pvUserContext, unsigned int uiReasonForCall);

//! Typedef for the function pointer passed as the seventh parameter to adqWriteContinuousStart - The parameter 'uiReasonForCall' is a bit-coded combination of the constants ADQ_CALLED_FOR_... above

typedef unsigned int (__stdcall *ADQ_WRITE_CONTINUOUS_CALLBACK)(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiNumberOfValuesAcceptable, unsigned int* puiValues, unsigned int& uiNumberOfValuesWritten, unsigned int uiError, void* pvUserContext, unsigned int uiReasonForCall);

//! Typedef for the function pointer passed as the fifth parameter to adqAIReadContinuousStart - The parameter 'uiReasonForCall' is a bit-coded combination of the constants ADQ_CALLED_FOR_... above

typedef unsigned int (__stdcall *ADQ_AI_READ_CONTINUOUS_CALLBACK)(unsigned int uiBoardIndex, unsigned int uiModuleIndex, unsigned int uiNumberOfFramesAvailable, unsigned int* puiFrames, unsigned int uiError, void* pvUserContext, unsigned int uiReasonForCall);

//@}

#pragma pack(pop)

#endif